# -*- coding: utf-8 -*-
"""
Created on Wed Feb  8 10:16:17 2012

@author: micha     February 2020
"""
import os
import sys
import wx
import numpy as np
import pylab

from matplotlib import __version__ as matplotlibversion
from matplotlib.backends.backend_wxagg import (FigureCanvasWxAgg as FigCanvas,
                                    NavigationToolbar2WxAgg as NavigationToolbar)
import matplotlib as mpl
from matplotlib.figure import Figure

if sys.version_info.major == 3:
    from .. import generaltools as GT
    from . PlotRefineGUI import IntensityScaleBoard
else:
    import generaltools as GT
    from GUI.PlotRefineGUI import IntensityScaleBoard

DEG = np.pi / 180.0
CA = np.cos(40.0 * DEG)
SA = np.sin(40.0 * DEG)


class SimulationPlotFrame(wx.Frame):
    r"""
    class to plot simulated Laue pattern of one or several grains
    """
    def __init__(self, parent, _id, title, data=(1, 1, 1, 1, 1, "2thetachi", None),
                                                                    ImageArray=None,
                                                                    StreakingData=None,
                                                                    list_grains_transforms=None,
                                                                    dirname=os.curdir,
                                                                    CCDLabel="MARCCD165",
                                                                    **kwds):
        r"""
        Class for 2D plot of spot location and get info from data

        Each element of data contains list composed of data set (e.g. corresponding to one grain)
        - [0] list of list of first coordinate of spots
        - [1] list of list of second coordinate of spots
        - [2] list of list of scalar that can be spot intensity or spot Energy
        - [3] list of list of 3d vector (3 integers defining miller indices)
        - [4] scalar giving the number of data set in the lists above (e.g. number of grains)
        - [5] string defining the type of data and coordinates (angular or cartesian) ('2thetachi','XY')
        - [6] 3-tuple for additional data to plot (e.g. experimental data):
            - [0] list of 1rst coordinates spots
            - [1] list of 2nd coordinates spots
            - [2] scalar (e.g. spot intensities)
        """
        wx.Frame.__init__(self, parent, _id, title, **kwds)

        self.SetSize(wx.Size(600, 1200))

        self.data = data

        # nb_grains = self.data[4]

        self.showFluoDetectorFrame = False
        self.showFluoDetectorFrameTools = False
        self.datatype = self.data[5]
        if self.datatype.endswith('fluo'):
            self.showFluoDetectorFrameTools = True
            self.showFluoDetectorFrame = True

        self.ImageArray = ImageArray
        self.data_dict = {}
        self.data_dict["Imin"] = 1
        self.data_dict["Imax"] = 1000
        self.data_dict["vmin"] = 1
        self.data_dict["vmax"] = 1000
        self.data_dict["lut"] = "jet"
        self.data_dict["logscale"] = True
        self.data_dict["markercolor"] = "b"
        self.data_dict["CCDLabel"] = CCDLabel

        self.X_offsetfluoframe = 0
        self.Y_offsetfluoframe = 0

        self.ScatterPlot_ParentGrain = {}
        self.ScatterPlot_Grain = {}
        transformindex = 0
        self.list_grains_transforms = list_grains_transforms
        nb_ParentGrains = len(self.list_grains_transforms)
        print("nb_ParentGrains", nb_ParentGrains)
        for parentgrainindex in range(nb_ParentGrains):
            print("self.list_grains_transforms", self.list_grains_transforms[parentgrainindex])
            _, nb_transforms, transform_type = self.list_grains_transforms[parentgrainindex]
            if transform_type in ("slipsystem",):
                self.ScatterPlot_ParentGrain[parentgrainindex] = False
            else:
                self.ScatterPlot_ParentGrain[parentgrainindex] = True
            # nb_transforms  = nb of subgrains
            # with slipsystem simulation:
            # nb_transforms = nb of steps (or subgrains) / slip * nb of slips
            for _ in range(nb_transforms):
                # print( "parentgrainindex,transformindex", parentgrainindex, transformindex )
                if transform_type in ("slipsystem",):
                    self.ScatterPlot_Grain[transformindex] = False
                else:
                    self.ScatterPlot_Grain[transformindex] = True
                transformindex += 1

        # print("ScatterPlot_ParentGrain", self.ScatterPlot_ParentGrain)
        # print("self.ScatterPlot_Grain", self.ScatterPlot_Grain)

        # StreakingData = data_res, SpotIndexAccum_list, GrainParent_list, TransformType_list, slipsystemsfcc
        self.StreakingData = StreakingData

        self.CCDLabel = CCDLabel

        self.init_plot = True

        # in plotRefineGUI
        #         self.data_theo = [Twicetheta, Chi, Miller_ind]
        #         self.data_theo_pixXY = [posx, posy, Miller_ind]

        self.dirname = dirname

        self.pick_distance_mode = False
        self.points = []  # to store points
        self.selectionPoints = []
        self.twopoints = []
        self.nbclick = 1
        self.nbclick_dist = 0

        # ----------    GUI ------------------------
        self.panel = wx.Panel(self)

        self.dpi = 100
        self.figsizex, self.figsizey = 5, 3
        self.fig = Figure((self.figsizex, self.figsizey), dpi=self.dpi)
        self.fig.set_size_inches(self.figsizex, self.figsizey, forward=True)
        self.canvas = FigCanvas(self.panel, -1, self.fig)

        self.axes = self.fig.add_subplot(111)

        self.toolbar = NavigationToolbar(self.canvas)

        self.angulardist_btn = wx.Button(self.panel, -1, "GetAngularDistance")
        self.pixeldist_btn = wx.Button(self.panel, -1, "GetPixelDistance")
        # self.pointButton3 = wx.ToggleButton(self.panel, 3, 'Show indices')
        self.drawindicesBtn = wx.ToggleButton(self.panel, 4, "Draw indices")
        self.setImageScalebtn = wx.Button(self.panel, -1, "Set Image && Scale")
        #  self.pointButton5 = wx.Button(self.panel, 5, 'Save Plot')
        self.pointButton6 = wx.Button(self.panel, -1, "Quit")
        self.defaultColor = self.GetBackgroundColour()
        # self.Bind(wx.EVT_BUTTON, self.OnSavePlot, id=5)
        self.pointButton6.Bind(wx.EVT_BUTTON, self.OnQuit)
        self.setImageScalebtn.Bind(wx.EVT_BUTTON, self.onSetImageScale)
        self.angulardist_btn.Bind(wx.EVT_BUTTON, self.GetAngularDistance)
        self.pixeldist_btn.Bind(wx.EVT_BUTTON, self.GetCartesianDistance)

        self.txtctrldistance_angle = wx.StaticText(self.panel, -1, "==> %s deg" % "", size=(80, -1))
        self.txtctrldistance_pixel = wx.StaticText(self.panel, -1, "==> %s pixel" % "", size=(80, -1))
        if self.datatype in ("pixels",) or self.showFluoDetectorFrameTools:
            self.sidefluodetector_btn = wx.Button(self.panel, -1, "FluoDetectorFrame")
            self.txtoffset = wx.StaticText(self.panel, -1, "Fluo frame origin")
            self.txtoffsetX = wx.StaticText(self.panel, -1, "X", size=(80, -1))
            self.offsetXtxtctrl = wx.TextCtrl(self.panel, -1, "0.0", size=(75, -1))

            self.txtoffsetY = wx.StaticText(self.panel, -1, "Y", size=(80, -1))
            self.offsetYtxtctrl = wx.TextCtrl(self.panel, -1, "0.0", size=(75, -1))

            self.sidefluodetector_btn.Bind(wx.EVT_BUTTON, self.OnSideFluoDetector)
            self.offsetXtxtctrl.Bind(wx.EVT_TEXT_ENTER, self.OnEnterOffsetX)
            self.offsetYtxtctrl.Bind(wx.EVT_TEXT_ENTER, self.OnEnterOffsetY)

        self.cidpress = self.fig.canvas.mpl_connect("button_press_event", self.onClick)

        self.tooltip = wx.ToolTip(tip="Welcome on LaueTools Laue Pattern simulation frame")
        self.canvas.SetToolTip(self.tooltip)
        self.tooltip.Enable(False)
        self.tooltip.SetDelay(0)
        self.fig.canvas.mpl_connect("motion_notify_event", self.onMotion_ToolTip)

        self.slider = wx.Slider(self.panel, -1, 50, 0, 100, size=(120, -1))
        self.slidertxt = wx.StaticText(self.panel, -1, "spot size", (5, 5))
        self.Bind(wx.EVT_SLIDER, self.sliderUpdate)

        self.statusBar = self.CreateStatusBar()

        self.readdata()

        print("factor spot size = %f " % self.factorsize)

        self._layout()

        self._replot()

        # for spot annotation
        self.drawnAnnotations = {}
        self.links = []

    def _layout(self):
        """arrange widgets
        """
        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(self.canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
        vbox.Add(self.toolbar, 0, wx.EXPAND)

        bottombarsizer = wx.BoxSizer(wx.HORIZONTAL)
        bottombarsizer.Add(self.slider)
        bottombarsizer.Add(self.slidertxt)

        btnSizer = wx.BoxSizer(wx.VERTICAL)
        btnSizer.Add(self.angulardist_btn, 0, wx.BOTTOM | wx.LEFT)
        btnSizer.Add(self.txtctrldistance_angle, 0, wx.BOTTOM | wx.LEFT)
        btnSizer.Add(self.pixeldist_btn, 0, wx.BOTTOM | wx.LEFT)
        btnSizer.Add(self.txtctrldistance_pixel, 0, wx.BOTTOM | wx.LEFT)
        # btnSizer.Add(self.pointButton3, 0, wx.BOTTOM | wx.LEFT)
        btnSizer.Add(self.drawindicesBtn, 0, wx.BOTTOM | wx.LEFT)
        # btnSizer.Add(self.pointButton5, 0, wx.BOTTOM | wx.LEFT)

        btnSizer.Add(self.setImageScalebtn, 0, wx.BOTTOM | wx.LEFT)
        btnSizer.Add(bottombarsizer, 0, wx.BOTTOM | wx.LEFT)
        btnSizer.Add(self.pointButton6, 0, wx.BOTTOM | wx.LEFT)
        if self.datatype in ("pixels",) or self.showFluoDetectorFrameTools:
            btnSizer.Add(self.sidefluodetector_btn, 0, wx.BOTTOM | wx.LEFT)

            btnSizer.Add(self.txtoffset, 0, wx.BOTTOM | wx.LEFT)

            b1 = wx.BoxSizer(wx.HORIZONTAL)
            b1.Add(self.txtoffsetX, 0, wx.BOTTOM | wx.LEFT)
            b1.Add(self.offsetXtxtctrl, 0, wx.BOTTOM | wx.LEFT)

            b2 = wx.BoxSizer(wx.HORIZONTAL)
            b2.Add(self.txtoffsetY, 0, wx.BOTTOM | wx.LEFT)
            b2.Add(self.offsetYtxtctrl, 0, wx.BOTTOM | wx.LEFT)

            btnSizer.Add(b1, 0, wx.BOTTOM | wx.LEFT)
            btnSizer.Add(b2, 0, wx.BOTTOM | wx.LEFT)

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(vbox, 1, wx.EXPAND)
        sizer.Add(btnSizer, 0, wx.EXPAND)

        self.panel.SetSizer(sizer)
        sizer.Fit(self)
        self.Layout()

    def sliderUpdate(self, _):
        # self.factorsize = np.exp(int(self.slider.GetValue()) / 500.) - 1.
        self.factorsize = 10 ** (
            int(self.slider.GetValue()) / 50.0 - 1.0 + np.log10(self.central_value))
        self._replot()
        # print "factor spot size = %f " % self.factorsize

    def onClick(self, event):
        """ onclick
        """
        print("self.nbclick_dist start onClick", self.nbclick_dist)
        #        print 'clicked on mouse'
        if event.inaxes:
            #            print("inaxes", event)
            #             print("inaxes x,y", event.x, event.y)
            print(("inaxes  xdata, ydata", event.xdata, event.ydata))
            self.centerx, self.centery = event.xdata, event.ydata

            if self.pick_distance_mode:
                self.nbclick_dist += 1
                print("self.nbclick_dist", self.nbclick_dist)

                if self.nbclick_dist == 1:

                    x, y = event.xdata, event.ydata

                    self.twopoints = [(x, y)]

                if self.nbclick_dist == 2:

                    x, y = event.xdata, event.ydata

                    self.twopoints.append((x, y))

                    spot1 = self.twopoints[0]
                    spot2 = self.twopoints[1]

                    if self.datatype == "2thetachi":
                        _dist = GT.distfrom2thetachi(np.array(spot1), np.array(spot2))
                        print("angular distance (q1,q2):  %.3f deg " % _dist)

                    if self.datatype == "gnomon":
                        from indexingImageMatching import Fromgnomon_to_2thetachi

                        tw, ch = Fromgnomon_to_2thetachi([
                                np.array([spot1[0], spot2[0]]),
                                np.array([spot1[1], spot2[1]])], 0)[:2]
                        _dist = GT.distfrom2thetachi(np.array([tw[0], ch[0]]),
                                                    np.array([tw[1], ch[1]]))
                        print("angular distance (q1,q2):  %.3f deg " % _dist)

                    if self.datatype == "pixels":
                        # dist in pixel when data are in pixel
                        _dist = np.sqrt((spot1[0] - spot2[0]) ** 2 + (spot1[1] - spot2[1]) ** 2)
                        # TODO: dist in pixel when data are in 2thetachi
                    #                         last_index = self.clicked_indexSpot[-1]
                    #                         print "last clicked", last_index
                    #                         if len(self.clicked_indexSpot) > 1:
                    #                             last_last_index = self.clicked_indexSpot[-2]
                    #                             print "former clicked", last_last_index
                    #
                    #                         spot1 = [self.tth[last_last_index], self.chi[last_last_index]]
                    #                         spot2 = [self.tth[last_index], self.chi[last_index]]
                    #
                    #                         _dist = GT.distfrom2thetachi(np.array(spot1),
                    #                                                      np.array(spot2))
                    #                         print "Angular distance (q1,q2):  %.3f deg " % _dist

                    # Reset for next pick request and display results
                    self.nbclick_dist = 0
                    # self.twopoints = []
                    self.pick_distance_mode = False
                    if self.datatype == "2thetachi":
                        print("RES =", _dist)
                        sentence = "Corresponding lattice planes angular distance"
                        sentence += ("\n between two scattered directions : %.2f " % _dist)
                        print(sentence)

                        self.angulardist_btn.SetBackgroundColour(self.defaultColor)
                        self.txtctrldistance_angle.SetLabel("==> %s deg" % str(np.round(_dist, decimals=3)))
                    if self.datatype == "pixels":
                        print("RES =", _dist)
                        sentence = "pixel distance"
                        sentence += ("\n between two scattered directions : %.2f " % _dist)
                        print(sentence)
                        self.pixeldist_btn.SetBackgroundColour(self.defaultColor)
                        self.txtctrldistance_pixel.SetLabel("==> %s pixels" % str(np.round(_dist, decimals=3)))

                    self.statusBar.SetStatusText(sentence, 0)

                    # DOES NOT WORK  stuck the computer!!
            #                     wx.MessageBox(sentence, 'INFO')

            #draw indices button !
            elif self.drawindicesBtn.GetValue():
                self.OnDrawIndices(event)

    def readdata(self):
        """
        read input parameter 'data'to be plotted
        """
        # if nb grains = 1

        # print self.data
        self.Data_X = self.data[0]
        self.Data_Y = self.data[1]
        self.Data_I = self.data[2]  # this column may contain energy
        self.Data_Miller = self.data[3]

        self.datatype = self.data[5]

        self.nbGrains = self.data[4]
        self.Data_index_expspot = np.arange(len(self.Data_X))

        self.experimentaldata = self.data[6]

        #         print "self.Data_I", self.Data_I
        #         print "self.Data_X", self.Data_X
        #         print "self.Data_Y", self.Data_Y

        # for many grains annotations, pixel X, pixel Y, intensity, Miller indices
        self.Xdat = []
        self.Ydat = []
        self.Idat = []
        self.Mdat = []
        self.grainindexdat = []
        self.parentgrainindexdat = []
        self.spotindex_in_grain = []
        # build data and list of spot index borders for each grain
        firstindex = 0
        lastindex = 0

        # defining limits of plot
        if self.nbGrains > 1:
            self.Xmin = min(list(map(min, self.Data_X)))
            self.Xmax = max(list(map(max, self.Data_X)))
            self.Ymin = min(list(map(min, self.Data_Y)))
            self.Ymax = max(list(map(max, self.Data_Y)))

            for k in range(self.nbGrains):
                #print("self.Data_X[k] ", self.Data_X[k])
                #                 print "len(self.data[0][k])", len(self.data[0][k])
                self.Xdat += self.Data_X[k]
                self.Ydat += self.Data_Y[k]
                self.Idat += self.Data_I[k]  # intensity or Energy
                self.Mdat += self.Data_Miller[k]  # miller indices
                lastindex = len(self.Data_X[k]) - 1
                self.spotindex_in_grain.append(firstindex)
                self.spotindex_in_grain.append(lastindex + firstindex)
                # print "firstindex",firstindex
                # print "lastindex+firstindex",lastindex+firstindex

                firstindex += lastindex + 1

            # tables containing infos on Laue spots / subgrain
            self.allspotindex = np.arange(len(self.Xdat))
            self.spotindex_in_grain = np.array(self.spotindex_in_grain)
            self.mini = self.spotindex_in_grain[::2]
            self.maxi = self.spotindex_in_grain[1::2]

            # spot size scale factor
            self.central_value = 100.0 / max(self.Idat) / 5.0

        elif self.nbGrains == 1:
            # print "rgtrg", self.Data_X[0]
            datX = self.Data_X[0]
            datY = self.Data_Y[0]
            self.Xmin = min(datX)
            self.Xmax = max(datX)
            self.Ymin = min(datY)
            self.Ymax = max(datY)

            self.Xdat = datX
            self.Ydat = datY
            self.Idat = self.Data_I[0]  # intensity or Energy
            self.Mdat = self.Data_Miller[0]

            self.allspotindex = np.arange(len(self.Xdat))
            self.mini = 0
            self.maxi = len(self.Xdat)

            # spot size scale factor
            # print "self.Idat", self.Idat
            self.central_value = 100.0 / max(self.Idat) / 5.0

        self.currentbounds = ((self.Xmin, self.Xmax), (self.Ymin, self.Ymax))
        self.factorsize = self.central_value
        # print "self.currentbounds", self.currentbounds

    def onMotion_ToolTip(self, event):
        """
        tool tip to show data when mouse hovers on plot on simulFrame
        """
        ExperimentalSpots = False

        if self.pick_distance_mode:
            return

        if len(self.data[0]) == 0:
            return

        collisionFound_exp = False
        collisionFound_theo = False

        if self.datatype in ("2thetachi",):
            xtol = 2
            ytol = 2
        else:
            xtol = 50
            ytol = 50

        if ExperimentalSpots:
            xdata, ydata, _annotes_exp = (self.Data_X,
                                            self.Data_Y,
                                            list(zip(self.Data_index_expspot, self.Data_I)))

        xdata, ydata, infos = self.Xdat, self.Ydat, list(zip(self.Idat, self.Mdat))
        #         print "print xdata", xdata
        #         print "print ydata", ydata
        #         print self.Idat
        #         print self.Mdat
        # print infos
        _dataANNOTE = list(zip(xdata, ydata, infos, self.allspotindex))

        clickX = event.xdata
        clickY = event.ydata

        #         print "clickX, clickY", clickX, clickY

        if event.xdata != None and event.ydata != None:

            clickX = event.xdata
            clickY = event.ydata

            #             print 'clickX,clickY in onMotion_ToolTip', clickX, clickY

            dataabscissa_name, dataordinate_name = "X", "Y"
            if self.datatype in ("2thetachi",):
                dataabscissa_name, dataordinate_name = "2theta", "chi"

            sttext = "(%s,%s)=(%.1f,%.1f) " % (dataabscissa_name, dataordinate_name, clickX, clickY)
            if self.showFluoDetectorFrame:
                sttext += "(ydet,zdet)= (%.1f,%.1f) " % self.convertXY2ydetzdet(clickX, clickY)

            self.statusBar.SetStatusText((sttext), 0)

            if ExperimentalSpots:
                annotes_exp = []
                for x, y, a in zip(xdata, ydata, _annotes_exp):
                    if (clickX - xtol < x < clickX + xtol) and (clickY - ytol < y < clickY + ytol):
                        annotes_exp.append((GT.cartesiandistance(x, clickX, y, clickY), x, y, a))

                if annotes_exp != []:
                    collisionFound_exp = True

            list_close_pts = []
            for x, y, a, ind in _dataANNOTE:
                if (clickX - xtol < x < clickX + xtol) and (clickY - ytol < y < clickY + ytol):
                    list_close_pts.append((GT.cartesiandistance(x, clickX, y, clickY), x, y, a, ind))

            if list_close_pts:
                list_close_pts.sort()
                # closest pt
                _distance, x, y, annote, ind = list_close_pts[0]

                global_spot_index = ind

                infostuple = self.getsubgraininfos(global_spot_index)
                print("the nearest simulated point is at (%.2f,%.2f)" % (x, y))
                print("with E= %.3f keV and Miller indices %s" % (annote[0], annote[1:]))
                # print("index : %d" % ind)
                # print('infostuple', str(infostuple))

                if not isinstance(self.maxi, int):
                    grain_index = np.searchsorted(self.maxi, global_spot_index)
                    first_grainindex = self.mini[grain_index]
                else:
                    grain_index = 0
                    first_grainindex = 0

                local_spot_index = global_spot_index - first_grainindex

            if list_close_pts != []:
                collisionFound_theo = True

            if not collisionFound_exp and not collisionFound_theo:
                return

            tip_exp = ""
            tip_theo = ""
            if collisionFound_exp:
                annotes_exp.sort()
                #                 print 'annotes_exp', annotes_exp
                _distance, x, y, annote_exp = annotes_exp[0]
                #             print "the nearest experimental point is at(%.2f,%.2f)" % (x, y)
                #             print "with index %d and intensity %.1f" % (annote[0], annote[1])

                # if exp. spot is close enough
                if _distance < xtol:
                    tip_exp = "spot index=%d. Intensity=%.1f" % (annote_exp[0], annote_exp[1])
                    self.updateStatusBar_theo_exp(x, y, annote_exp, spottype="exp")
                else:
                    self.sb.SetStatusText("", 1)
                    collisionFound_exp = False

            if collisionFound_theo:
                # if theo spot is close enough
                if _distance < xtol:
                    E = annote[0]
                    HKL = annote[1:4]

                    tip_theo = " SPOT @ x= %.2f y= %2f E= %.3f [h,k,l]=%s" % (x, y, E, HKL)

                    subgrainindex, parentgrainindex, _ = infostuple
                    #starting subgrains index:
                    stindex = self.getstartingsubgrainindex(subgrainindex) # val[0]

                    tip_theo += ' Grain_%d, subgrain_%d'%(parentgrainindex, subgrainindex)
                    tip_theo += " spotindex in grain: %d " % local_spot_index

                    if 'Slips' in self.datatype:
                        # print("Data for slips")
                        # Assuming a single slip system simulation
                        list_ParentGrain_transforms = self.StreakingData[0][7]
                        # print("list_ParentGrain_transforms",list_ParentGrain_transforms)

                        nbsteps = 11
                        slipindex = (grain_index - stindex) // nbsteps

                        if list_ParentGrain_transforms[parentgrainindex][2] == 'slipsystem':
                            plane, direction = self.StreakingData[4][slipindex]
                            tip_theo += '\nslipsystem infos: index %d' % slipindex
                            tip_theo += ': plane %s, direction %s'%(str(plane), str(direction))

                    sttext += tip_theo

                    self.statusBar.SetStatusText((sttext), 0)

                else:
                    self.statusBar.SetStatusText("")
                    collisionFound_theo = False

            if collisionFound_exp or collisionFound_theo:
                if tip_exp is not "":
                    fulltip = tip_exp + "\n" + tip_theo
                else:
                    fulltip = tip_theo

                self.tooltip.SetTip(fulltip)
                self.tooltip.Enable(True)
                return

        if not collisionFound_exp and not collisionFound_theo:
            pass

    def getsubgraininfos(self, spotindex):
        """
        from spotindex return subgrainindex, grainparentindex, transform_type
        StreakingData = data_res, SpotIndexAccum_list, GrainParent_list, TransformType_list, slipsystemsfcc
        """
        subg = np.array(self.StreakingData[1]).searchsorted(spotindex)
        GrPar = self.StreakingData[2][subg]
        transformtype = self.StreakingData[3][subg]
        return subg, GrPar, transformtype

    def getstartingsubgrainindex(self, subgrainindex):
        """ return first subgrain index of a set of grains which contains subgrain 'subgrainindex'
        """

        GrParList = self.StreakingData[2]
        GrPar = GrParList[subgrainindex]

        return GrParList.index(GrPar)

    def func_size_energy(self, val, factor):
        return 400.0 * factor / (val + 1.0)

    def func_size_intensity(self, val, factor, offset, lin=1):
        if lin:
            return factor * val + offset
        else:  # log scale
            return factor * np.log(np.clip(val, 0.000000001, 1000000000000)) + offset

    def fromindex_to_pixelpos_x(self, index, _):
        """
        x ticks format
        """
        return index

    def fromindex_to_pixelpos_y(self, index, _):
        """
        y ticks format
        """
        return index

    def setplotlimits_fromcurrentplot(self):
        self.xlim = self.axes.get_xlim()
        self.ylim = self.axes.get_ylim()
        #print("new limits x", self.xlim)
        #print("new limits y", self.ylim)

    def OnSideFluoDetector(self, _):
        """
        change frame of pixel to be see x ray like side fluorescence detector

        but first show frame of ydet and zdet
        """
        self.showFluoDetectorFrame = not self.showFluoDetectorFrame
        # in side Y+ frame
        if self.showFluoDetectorFrame:
            self.X_offsetfluoframe = float(self.offsetXtxtctrl.GetValue())
            self.Y_offsetfluoframe = float(self.offsetYtxtctrl.GetValue())
        self._replot()

    def OnEnterOffsetX(self, _):
        """
        set the origin of ydet zdet fluodetector frame in pixel x,y frame
        """
        self.X_offsetfluoframe = float(self.offsetXtxtctrl.GetValue())  # in pixel
        self._replot()

    def OnEnterOffsetY(self, _):
        """
        set the origin of ydet zdet fluodetector frame in pixel x,y frame
        """
        self.Y_offsetfluoframe = float(self.offsetYtxtctrl.GetValue())  # in pixel
        self._replot()

    def convertXY2ydetzdet(self, X, Y):

        #         RotYm40 = np.array([[ca, sa], [-sa, ca]])
        Xo = X - self.X_offsetfluoframe
        Yo = Y - self.Y_offsetfluoframe
        ydet = -CA * Xo + -SA * Yo
        zdet = -(SA * Xo - CA * Yo)

        return ydet, zdet

    def _replot(self):
        """
        draw scatter plot of spots in SimulationPlotFrame()
        """
        print("self.datatype", self.datatype)
        print("self.init_plot", self.init_plot)
        # offsets to match imshow and scatter plot coordinates frames
        # coordinates in pixels
        if 'XYMAR' in self.datatype:
            self.X_offset = 1
            self.Y_offset = 1
        #coordinates 2theta chi  or gnomon
        else:
            self.X_offset = 0
            self.Y_offset = 0

        if not self.init_plot:
            self.setplotlimits_fromcurrentplot()

        self.axes.clear()
        self.axes.set_autoscale_on(False)  # Otherwise, infinite loop

        self.axes.xaxis.set_major_formatter(pylab.FuncFormatter(self.fromindex_to_pixelpos_x))
        self.axes.yaxis.set_major_formatter(pylab.FuncFormatter(self.fromindex_to_pixelpos_y))

        if self.showFluoDetectorFrame:
            print("show Fluodetector frame in side")
            # do not forget to set pixelsize = 1 (mm) and detector distance 50 mm
            # xcen ycen xet xgam = 0
            # this configuration corresponds to laue spots coming from the sample and hitting the observer
            # and the fluo detector. zdet up (tilted by 40 deg to the right)
            # and ydet to the right (tilted by 40 deg) and going down
            
            # not in Marccd big nb of pixels convention
            self.X_offset = 0
            self.Y_offset = 0

            if self.init_plot:
                self.ylim = (-200, 200)
                self.xlim = (200, -200)
            
            center_detframe = [self.X_offsetfluoframe, self.Y_offsetfluoframe]

            # range rectangle in ydet et zdet frame
            pt_lb = [-40.0, 0.0]
            pt_lt = [-40.0, 30.0]
            pt_rt = [40.0, 30.0]
            pt_rb = [40.0, 0.0]
            # 40 deg tilt of detector motors frame
            RotY40 = np.array([[CA, -SA], [SA, CA]])

            rot_pts = np.dot(RotY40, np.array([pt_lb, pt_lt, pt_rt, pt_rb]).T)
            trans_rot_pts = rot_pts.T + np.array(center_detframe)

            pt_lb_prime, pt_lt_prime, pt_rt_prime, pt_rb_prime = trans_rot_pts

            pt_zaxis_prime = (pt_lt_prime + pt_rt_prime) / 2.0

            from matplotlib.collections import LineCollection
            import matplotlib.patches as patches
            if sys.version_info.major == 3:
                from .. import rectangle as rect
            else:
                import rectangle as rect

            w_nbsteps = 40
            h_nbsteps = 15

            segs_vert, segs_hor = rect.getsegs_forlines_2(pt_lb_prime, pt_rb_prime, pt_lt_prime,
                                                            w_nbsteps, h_nbsteps)

            line_segments_vert = LineCollection(segs_vert, linestyle="solid")
            self.axes.add_collection(line_segments_vert)

            line_segments_hor = LineCollection(segs_hor, linestyle="solid", colors="r")
            self.axes.add_collection(line_segments_hor)

            # arrows = [patches.YAArrow(self.fig, (pt_lb_prime[0], pt_lb_prime[1]),
            #                 (center_detframe[0], center_detframe[1]), fc="g",
            #                 width=0.3, headwidth=0.9, alpha=0.5),
            #         patches.YAArrow(self.fig, (pt_zaxis_prime[0], pt_zaxis_prime[1]),
            #                 (center_detframe[0], center_detframe[1]), fc="r",
            #                 width=0.3, headwidth=0.9, alpha=0.5)]

            # Ydetarrow
            x1b, y1b = (center_detframe[0], center_detframe[1])  # base
            x1t, y1t = (pt_lb_prime[0], pt_lb_prime[1])  # tip

            # Zdetarrow
            #x2b, y2b = (center_detframe[0], center_detframe[1])  # base
            x2t, y2t = (pt_zaxis_prime[0], pt_zaxis_prime[1])  # tip
            
            arrows = [patches.FancyArrow(x1b, y1b, x1t - x1b, y1t - y1b, fc="g",
                            width=0.3, head_width=0.9, alpha=0.5),
                    patches.FancyArrow(x1b, y1b, x2t - x1b,y2t - y1b, fc="r",
                            width=0.3, head_width=0.9, alpha=0.5)]

            for ar in arrows:
                self.axes.add_patch(ar)

        if self.ImageArray is not None:

            print("self.ImageArray", self.ImageArray.shape)
            self.myplot = self.axes.imshow(self.ImageArray, interpolation="nearest")

            if not self.data_dict["logscale"]:
                norm = mpl.colors.Normalize(vmin=self.data_dict["vmin"], vmax=self.data_dict["vmax"])
            else:
                norm = mpl.colors.LogNorm(vmin=self.data_dict["vmin"], vmax=self.data_dict["vmax"])

            self.myplot.set_norm(norm)
            self.myplot.set_cmap(self.data_dict["lut"])
            kwords = {"marker": "o", "facecolor": "None"}

        else:
            kwords = {"edgecolor": "None"}

        if self.nbGrains > 1:
            colors = GT.JET(np.arange(self.nbGrains) * 1.0 / ((self.nbGrains - 1)))
        elif self.nbGrains == 1:
            colors = [list(GT.JET(0))]

        #---------------------------------------
        # loop over grains => scatter plot
        #---------------------------------------
        print('self.nbGrains', self.nbGrains)
        for grainindex in range(self.nbGrains):

            # slip systems ---------------------
            print('grainindex %d , self.ScatterPlot_Grain[grainindex]' % grainindex, self.ScatterPlot_Grain[grainindex])
            if not self.ScatterPlot_Grain[grainindex]:
                continue

            # print "self.Data_X[grainindex] in plot", self.Data_X[grainindex]
            colors[grainindex][3] = 0.0  # set alpha to 0  i.e. full transparency

            if self.ImageArray is not None:
                kwords["edgecolors"] = tuple(colors[grainindex])
            else:
                kwords["c"] = tuple(colors[grainindex])

            print('np.array(self.Data_X[grainindex])',np.array(self.Data_X[grainindex]))

            # theo Laue spots scatter plot
            self.axes.scatter(np.array(self.Data_X[grainindex]) - self.X_offset,
                            np.array(self.Data_Y[grainindex]) - self.Y_offset,
                            s=self.func_size_energy(
                                np.array(self.Data_I[grainindex]), self.factorsize
                            ), alpha=1.0,
                            **kwords)

        # plot lines to connect Laue spots for each slip
        if 'Slips' in self.datatype:# and self.isSingleStreakingPlot():
            print('\n**************\nEntering Slips line plot\n************\n')

            # self.StreakingData[0] = (
            #list_twicetheta, list_chi, list_energy, list_Miller,
            #list_posX, list_posY,
            #ParentGrainName_list, list_ParentGrain_transforms, calib, total_nb_grains, )

            ParentGrainName_list = self.StreakingData[0][6]
            list_ParentGrain_transforms = self.StreakingData[0][7]

            print('ParentGrainName_list', ParentGrainName_list)
            print('list_ParentGrain_transforms', list_ParentGrain_transforms)

            dictindicesStreakingData = getindices_StreakingData(list_ParentGrain_transforms)
            print('dictindicesStreakingData', dictindicesStreakingData)

            for elem in list_ParentGrain_transforms:
                parentgrainindex, _, _ = elem
                print('parentgrainindex: %d, self.ScatterPlot_ParentGrain[parentgrainindex]'%parentgrainindex,
                                            self.ScatterPlot_ParentGrain[parentgrainindex])

                if self.ScatterPlot_ParentGrain[parentgrainindex]:
                    continue

                allrawX = self.StreakingData[0][4]
                allrawY = self.StreakingData[0][5]

                print('len(allrawX)', len(allrawX))

                sindex, findex = dictindicesStreakingData[parentgrainindex]
                rawX = np.array(allrawX[sindex:findex])
                rawY = np.array(allrawY[sindex:findex])

                print('rawX.shape', rawX.shape)
                nbsubgrains, nbLauespots = rawX.shape

                slipsystem = self.StreakingData[4]
                nbslips = len(slipsystem)
                nbsteps = nbsubgrains//nbslips

                print("nbLauespots", nbLauespots)
                print('nbsubgrains', nbsubgrains)
                print('nbsteps = nbsubgrains/slip', nbsteps)
                print('nbslips', nbslips)

                try:
                    XX = rawX.T.reshape((nbLauespots, nbslips, nbsteps))
                    YY = rawY.T.reshape((nbLauespots, nbslips, nbsteps))
                except ValueError:
                    wx.MessageBox('Sorry!\nYou still cannot mix a slip system simulation with a single crystal one', 'In DEVELOPEMENT')


                # print(XX[0])
                # print(XX[0,0])

                colorsslip = GT.JET(np.arange(nbslips) * 1.0 / ((nbslips-1)))
                # colorsslip[parentgrainindex][3] = 0.0  # set alpha to 0  i.e. full transparency
                kwords_slip = {}

                # --- add lines between extreme Laue spots for each slip

                for spotindex in range(nbLauespots):
                    for slipindex in range(nbslips):

                        xx = XX[spotindex][slipindex]
                        yy = YY[spotindex][slipindex]
                        kwords_slip["c"] = tuple(colorsslip[slipindex])
                        s = 0
                        mid = nbsteps//2
                        e = -1
                        if spotindex == 0:
                            print('[xx[s], xx[mid], xx[e]]', [xx[s], xx[mid], xx[e]])
                            print('slipindex', slipindex)
                        self.axes.plot(np.array([xx[s], xx[mid], xx[e]]) - self.X_offset,
                                        np.array([yy[s], yy[mid], yy[e]]) - self.Y_offset,
                                        '-o', **kwords_slip)
        #---------------------------------

        if self.init_plot:
            print('self.datatype', self.datatype)
            if self.datatype == "2thetachi":
                self.axes.set_xlabel("2theta (deg.)")
                self.axes.set_ylabel("chi (deg)")
                self.ylim = (-50, 50)
                self.xlim = (40, 130)
            elif self.datatype == "gnomon":
                self.axes.set_xlabel("xgnomon (deg.)")
                self.axes.set_ylabel("ygnomon (deg)")
                self.ylim = (-.75, .75)
                self.xlim = (-.6, .6)
            elif "XYmar" in self.datatype:

                self.axes.set_xlabel("X (pixel)")
                self.axes.set_ylabel("Y (pixel)")

                if self.showFluoDetectorFrame:
                    
                    self.ylim = (-200, 200)
                    self.xlim = (200, -200)
                # marccd and roper convention
                elif self.CCDLabel in ("MARCCD165", "PRINCETON"):
                    self.ylim = (2048, 0)
                    self.xlim = (0, 2048)
                elif self.CCDLabel in ("sCMOS", "sCMOS_fliplr"):
                    self.ylim = (2100, 0)
                    self.xlim = (0, 2100)
                elif self.CCDLabel in ("VHR_PSI",):
                    self.ylim = (3000, 0)
                    self.xlim = (0, 4000)
                elif self.CCDLabel in ("EIGER_4M",):
                    self.ylim = (2200, 0)
                    self.xlim = (0, 2200)
                elif self.CCDLabel in ("EIGER_1M",):
                    self.ylim = (1100, 0)
                    self.xlim = (0, 1100)
                elif self.CCDLabel in ("EDF",):
                    self.ylim = (1100, 0)
                    self.xlim = (0, 1100)
                else:
                    wx.MessageBox('The camera with label "%s" is not implemented yet' % self.CCDLabel,
                                    "info",)

        self.axes.set_title("number of grain(s) : %s" % self.nbGrains)
        self.axes.grid(True)

        if self.experimentaldata:  # plot experimental data
            self.axes.scatter(
                self.experimentaldata[0] - self.X_offset,
                self.experimentaldata[1] - self.Y_offset,
                s=self.func_size_intensity(np.array(self.experimentaldata[2]), 0.02, 0.0, lin=1),
                                                facecolor="r",
                                                edgecolor="r",
                                                marker="+")

        # restore the zoom limits (unless they're for an empty plot)
        if self.xlim != (0.0, 1.0) or self.ylim != (0.0, 1.0):
            self.axes.set_xlim(self.xlim)
            self.axes.set_ylim(self.ylim)

        self.init_plot = False
        # redraw the display
        self.canvas.draw()

    def isSingleStreakingPlot(self):
        flag = True
        # ParentGrainName_list = self.StreakingData[0][6]
        list_ParentGrain_transforms = self.StreakingData[0][7]

        # print('ParentGrainName_list', ParentGrainName_list)
        print('list_ParentGrain_transforms', list_ParentGrain_transforms)

        # disable slip system in case of mixture: single crystals and slip system simulation
        for elem in list_ParentGrain_transforms:
            _, _, transform_type = elem
            if 'param' in transform_type:
                flag = False
                break
        return flag

    def OnSavePlot(self, _):
        dlg = wx.FileDialog(self, "Saving in png format. Choose a file", self.dirname, "", "*.*",
            wx.SAVE | wx.OVERWRITE_PROMPT)
        if dlg.ShowModal() == wx.ID_OK:
            # Open the file for write, write, close
            filename = dlg.GetFilename()
            # dirname = dlg.GetDirectory()

            if len(str(filename).split(".")) > 1:  # filename has a extension
                Pre, Ext = str(filename).split(".")
                if Ext != "png":
                    filename = Pre + ".png"
            else:
                filename = filename + ".png"

            self.fig.savefig(os.path.join(dirname, filename), dpi=300)

        dlg.Destroy()

    def OnQuit(self, _):
        self.Destroy()

    def updateStatusBar(self, x, y, annote, grain_index, local_spot_index):

        E = annote[0]
        HKL = annote[1:4]

        self.statusBar.SetStatusText(
            ("grainindex: %d local spotindex: %d " % (grain_index, local_spot_index)
                + "x= %.2f " % x
                + "y= %2f " % y
                + "E=%.5f " % E
                + "HKL=%s" % HKL), 0)

    def OnDrawIndices(self, event):
        """
        OnDrawIndices in SimulationPlotFrame
        """
        xtol = 20
        ytol = 20

        #         print "self.axes.viewLim.bounds", self.axes.viewLim.bounds
        #         print "self.axes.get_xbound", self.axes.get_xbound()
        #         print "self.axes.get_ybound", self.axes.get_ybound()

        self.currentbounds = (self.axes.get_xbound(), self.axes.get_ybound())

        xdata, ydata, infos = self.Xdat, self.Ydat, list(zip(self.Idat, self.Mdat))
        #         print "print xdata", xdata
        #         print "print ydata", ydata
        #         print self.Idat
        #         print self.Mdat
        # print infos
        self._dataANNOTE = list(zip(xdata, ydata, infos, self.allspotindex))

        clickX = event.xdata
        clickY = event.ydata

        #         print "clickX, clickY", clickX, clickY

        list_close_pts = []
        for x, y, a, ind in self._dataANNOTE:
            if (clickX - xtol < x < clickX + xtol) and (clickY - ytol < y < clickY + ytol):
                list_close_pts.append((GT.cartesiandistance(x, clickX, y, clickY), x, y, a, ind))

        if list_close_pts:
            list_close_pts.sort()
            # closest pt
            _distance, x, y, annote, ind = list_close_pts[0]
            print("the nearest simulated point is at (%.2f,%.2f)" % (x, y))
            print("with E= %.3f keV and Miller indices %s" % (annote[0], annote[1:]))
            print("index : %d" % ind)

            global_spot_index = ind

            #             print 'self.maxi', self.maxi
            #             print 'self.mini', self.mini

            if not isinstance(self.maxi, int):
                grain_index = np.searchsorted(self.maxi, global_spot_index)
                first_grainindex = self.mini[grain_index]
            else:
                grain_index = 0
                first_grainindex = 0

            local_spot_index = global_spot_index - first_grainindex

            self.updateStatusBar(x, y, annote, grain_index, local_spot_index)
            self.drawAnnote(self.axes, x, y, annote)
            for l in self.links:
                l.drawSpecificAnnote(annote)
        else:
            print("you clicked too far from any spot!")

    def drawAnnote(self, axis, x, y, annote):
        """
        Draw the annotation on the plot
        """
        if (x, y) in self.drawnAnnotations:
            markers = self.drawnAnnotations[(x, y)]
            # print markers
            for m in markers:
                m.set_visible(not m.get_visible())

        else:
            coef_text_offset = 5
            # t = axis.text(x, y, "(%3.2f, %3.2f) - %s"%(x, y,annote), ) # par defaut
            t1 = axis.text(x + coef_text_offset * 1, y + coef_text_offset * 3,
                "%.3f" % (annote[0]),
                size=8)
            t2 = axis.text(x + coef_text_offset * 1, y - coef_text_offset * 3,
                "%d %d %d" % (int(annote[1][0]), int(annote[1][1]), int(annote[1][2])),
                size=8)

            m = axis.scatter([x], [y], s=1, marker="d", c="r", zorder=100, edgecolors="None",
                alpha=0.5)  # matplotlib 0.99.1.1

            self.drawnAnnotations[(x, y)] = (t1, t2, m)

        self.canvas.draw()

    def drawSpecificAnnote(self, annote):
        annotesToDraw = [(x, y, a) for x, y, a in self._dataANNOTE if a == annote]
        for x, y, a in annotesToDraw:
            self.drawAnnote(self.axes, x, y, a)

    def drawAnnote_exp(self, axis, x, y, annote):
        """
        Draw the annotation on the plot here it s exp spot index
        #from Plot_RefineFrame
        """
        if (x, y) in self.drawnAnnotations_exp:
            markers = self.drawnAnnotations_exp[(x, y)]
            # print markers
            for m in markers:
                m.set_visible(not m.get_visible())
            # self.axis.figure.canvas.draw()
            #            self.plotPanel.draw()
            self.canvas.draw()
        else:
            # t = axis.text(x, y, "(%3.2f, %3.2f) - %s"%(x, y,annote), )  # par defaut
            t1 = axis.text(x + 1, y + 1, "#spot %d" % (annote[0]), size=8)
            t2 = axis.text(x + 1, y - 1, "Intens. %.1f" % (annote[1]), size=8)
            if matplotlibversion <= "0.99.1":
                m = axis.scatter([x], [y], s=1, marker="d", c="r", zorder=100, faceted=False)
            else:
                m = axis.scatter([x], [y], s=1, marker="d", c="r", zorder=100, edgecolors="None")  # matplotlib 0.99.1.1
            self.drawnAnnotations_exp[(x, y)] = (t1, t2, m)
            # self.axis.figure.canvas.draw()
            #            self.plotPanel.draw()
            self.canvas.draw()

    def drawSpecificAnnote_exp(self, annote):
        """
        from Plot_RefineFrame
        """
        annotesToDraw = [(x, y, a) for x, y, a in self._dataANNOTE_exp if a == annote]
        for x, y, a in annotesToDraw:
            self.drawAnnote_exp(self.axes, x, y, a)

    def Annotate_exp(self, event):
        """
        from Plot_RefineFrame
        """
        xtol = 20
        ytol = 20
       
        xdata, ydata, annotes = (self.Data_X, self.Data_Y,
            list(zip(self.Data_index_expspot, self.Data_I)))
       
        self._dataANNOTE_exp = list(zip(xdata, ydata, annotes))

        clickX = event.xdata
        clickY = event.ydata

        # print clickX, clickY

        annotes = []
        for x, y, a in self._dataANNOTE_exp:
            if (clickX - xtol < x < clickX + xtol) and (clickY - ytol < y < clickY + ytol):
                annotes.append((GT.cartesiandistance(x, clickX, y, clickY), x, y, a))

        if annotes:
            annotes.sort()
            _distance, x, y, annote = annotes[0]
            print("the nearest experimental point is at(%.2f,%.2f)" % (x, y))
            print("with index %d and intensity %.1f" % (annote[0], annote[1]))
            self.drawAnnote_exp(self.axes, x, y, annote)
            for l in self.links_exp:
                l.drawSpecificAnnote_exp(annote)

            self.updateStatusBar_theo_exp(x, y, annote, spottype="exp")

    def drawAnnote_theo(self, axis, x, y, annote):
        """
        Draw the annotation on the plot here it s exp spot index

        from Plot_RefineFrame
        """
        if (x, y) in self.drawnAnnotations_theo:
            markers = self.drawnAnnotations_theo[(x, y)]
            # print markers
            for m in markers:
                m.set_visible(not m.get_visible())
            # self.axis.figure.canvas.draw()
            #            self.plotPanel.draw()
            self.canvas.draw()
        else:
            # t = axis.text(x, y, "(%3.2f, %3.2f) - %s"%(x, y,annote), )  # par defaut
            t1 = axis.text(x - 7, y, "%s" % (str(annote)), size=8, color="r")

            if matplotlibversion <= "0.99.1":
                m = axis.scatter([x], [y], s=1, marker="d", c="r", zorder=100, faceted=False)
            else:
                m = axis.scatter([x], [y], s=1, marker="d", c="r", zorder=100, edgecolors="None")  # matplotlib 0.99.1.1

            self.drawnAnnotations_theo[(x, y)] = (t1, m)
            # self.axis.figure.canvas.draw()
            #            self.plotPanel.draw()
            self.canvas.draw()

    def drawSpecificAnnote_theo(self, annote):
        annotesToDraw = [(x, y, a) for x, y, a in self._dataANNOTE_theo if a == annote]
        for x, y, a in annotesToDraw:
            self.drawAnnote_theo(self.axes, x, y, a)

    def Annotate_theo(self, event):
        """ Display Miller indices of user clicked spot

        Add annotation in plot for the theoretical spot that is closest to user clicked point

        #from Plot_RefineFrame
        """
        xtol = 20.0
        ytol = 20.0

        if self.datatype == "2thetachi":
            xdata, ydata, annotes = self.data_theo  # 2theta chi miller

        elif self.datatype == "gnomon":
            xdata, ydata, annotes = self.data_theo  # sim_gnomonx, sim_gnomony, Miller_ind

        self._dataANNOTE_theo = list(zip(xdata, ydata, annotes))

        clickX = event.xdata
        clickY = event.ydata

        #         print "clickX, clickY", clickX, clickY

        annotes = []
        for x, y, a in self._dataANNOTE_theo:
            if (clickX - xtol < x < clickX + xtol) and (clickY - ytol < y < clickY + ytol):
                annotes.append((GT.cartesiandistance(x, clickX, y, clickY), x, y, a))

        if annotes:
            annotes.sort()
            _distance, x, y, annote = annotes[0]  # this the best of annotes
            # print("the nearest theo point is at(%.2f,%.2f)" % (x, y))
            # print("with index %s " % (str(annote)))
            self.drawAnnote_theo(self.axes, x, y, annote)
            for l in self.links_theo:
                l.drawSpecificAnnote_theo(annote)

            self.updateStatusBar_theo_exp(x, y, annote)

    def updateStatusBar_theo_exp(self, x, y, annote, spottype="theo"):

        if self.datatype == "2thetachi":
            Xplot = "2theta"
            Yplot = "chi"
        else:
            Xplot = "x"
            Yplot = "y"

        if spottype == "theo":
            self.sb.SetStatusText(("%s= %.2f " % (Xplot, x)
                    + " %s= %.2f " % (Yplot, y)
                    + "  HKL=%s " % str(annote)), 0)

        elif spottype == "exp":

            self.sb.SetStatusText(("%s= %.2f " % (Xplot, x)
                    + " %s= %.2f " % (Yplot, y)
                    + "   Spotindex=%d " % annote[0]
                    + "   Intensity=%.2f" % annote[1]), 1)

    def GetAngularDistance(self, _):
        """
        computes angle (in deg) between the NORMALS of TWO ATOMIC PLANES corresponding of
        the two clicked spots (NOT the angular distance between the spots)
        """
        if self.datatype != "2thetachi":
            sentence = "data are not in angular coordinate!"
            GT.printred(sentence)
            self.statusBar.SetForegroundColour(wx.RED)
            self.statusBar.SetStatusText("ERROR: %s" % sentence, 0)
            return

        self.pick_distance_mode = True
        self.angulardist_btn.SetBackgroundColour("Green")

        return

    def GetCartesianDistance(self, _):
        """
        computes angle (in deg) between the normals of atomic planes corresponding of
        the two clicked spot
        """
        if self.datatype != "pixels":
            print("data are not in pixel or cartesian coordinates!")
            return

        self.pick_distance_mode = True
        self.pixeldist_btn.SetBackgroundColour("Green")

        return

    def onSetImageScale(self, _):
        """
        open a board to open image and play with its intensity scale
        """
        if self.datatype == "2thetachi":
            wx.MessageBox('Please, open a new simulation window by restarting the simulation '
                                        'with XYPixel "in Display parameters" checked', 'Info')
            return

        IScaleBoard = IntensityScaleBoard(self, -1, "Image scale setting Board", self.data_dict)

        IScaleBoard.Show(True)

def getindices_StreakingData(list_ParentGrain_transforms):
    """read list_ParentGrain_transforms and return for each element
    the positions indices to extract date for straeking plot

    # assuming that first elements are in correct order and contiguous ...

    >>> getindices_StreakingData([[0, 132, 'slipsystem'], [1, 1, 'parametric']])
    >>> {0:[0,132],1:None}

    >>> getindices_StreakingData([[0,15,'parametric'],[1,132,'slipsystem'],
    [2,1,'parametric'],[3,500,'slipsystem']])
    >>> {0: None, 1: [15, 147], 2: None, 3: [148, 648]}


    """
    nbparentgrains = len(list_ParentGrain_transforms)
    dictindices = {}

    accum_nb = 0
    for k in range(nbparentgrains):
        gindex, nbtransforms, transform_type = list_ParentGrain_transforms[k]
        if transform_type == 'slipsystem':
            dictindices[gindex] = [accum_nb, accum_nb + nbtransforms]
        if transform_type == 'parametric':
            dictindices[gindex] = None
        accum_nb += nbtransforms

    return dictindices


if __name__ == "__main__":

    dirname = os.curdir

    data_xyI_1 = np.random.rand(3, 20).tolist()
    x_1, y_1, I_1 = data_xyI_1

    data_xyI_2 = np.random.rand(3, 35).tolist()
    x_2, y_2, I_2 = data_xyI_2

    miller_1 = np.array(np.random.rand(20, 3) * 10, dtype=np.uint8).tolist()
    miller_2 = np.array(np.random.rand(35, 3) * 10, dtype=np.uint8).tolist()
    nb = 2
    plottype = "2thetachi"
    exp_data = None

    data = [
        [x_1, x_2],
        [y_1, y_2],
        [I_1 * 10000, I_2 * 10000],
        [miller_1, miller_2],
        nb,
        plottype,
        exp_data,
    ]

    # data = [x_1, y_1, I_1 * 1000, miller_1, 1, datatype, exp_data]

    # print "data", data

    class App(wx.App):
        def OnInit(self):
            """Create the main window and insert the custom frame"""

            frame = SimulationPlotFrame(
                None, -1, "Laue Simulation Frame", data=data, dirname=dirname
            )
            self.SetTopWindow(frame)
            frame.Show(True)
            return True

    app = App(0)
    app.MainLoop()



import numpy as np
import math

import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
import matplotlib.patches as patches


# import matplotlib.backend_bases as aa

def twoindices_positive_up_to(n, m):
    """
    build  2D integer indices up to n
    """
    if not isinstance(n, int) or n <= 0:
        raise ValueError("%s is not a positive integer" % str(n))
        return None

    nbpos_n = n + 1
    nbpos_m = m + 1

    gripos = np.mgrid[:n:nbpos_n * 1j, :m:nbpos_m * 1j]
    indices_pos = np.reshape(gripos.T, (nbpos_n * nbpos_m, 2))

    return indices_pos


class Annotate(object):
    def __init__(self):
        self.ax = plt.gca()

        self.readdata()
        self.ax.imshow(self.data)

#         self.rect = Rectangle((0, 0), 1, 1)
        self.x0 = None
        self.y0 = None
        self.x1 = None
        self.y1 = None
#         self.ax.add_patch(self.rect)
        self.ax.figure.canvas.mpl_connect('button_press_event', self.on_press)
        self.ax.figure.canvas.mpl_connect('button_release_event', self.on_release)
        self.ax.figure.canvas.mpl_connect('motion_notify_event', self.on_motion)

        self.currentselection = False
        self.rectangleexists = False

    def on_press(self, event):
        if not self.rectangleexists:
            print('press')
            self.x0 = event.xdata
            self.y0 = event.ydata

            self.currentselection = True
            self.selectrect = Rectangle((self.x0, self.y0), 0.02, 0.02, facecolor='yellow', edgecolor='black')
            self.selectrect.set_linestyle('dashed')
            self.selectrect.set_alpha(0.2)
            self.ax.add_artist(self.selectrect)

    def on_release(self, event):
        if not self.rectangleexists:
            print('release')
            self.x1 = event.xdata
            self.y1 = event.ydata
            self.selectrect.set_width(self.x1 - self.x0)
            self.selectrect.set_height(self.y1 - self.y0)
            self.selectrect.set_xy((self.x0, self.y0))
            self.selectrect.set_linestyle('solid')
            self.ax.figure.canvas.draw()

            self.currentselection = False
            self.rectangleexists = True

    def on_motion(self, event):
        if not self.rectangleexists:
            print('on_motion')
            if self.currentselection is True and event.inaxes:
                self.x1 = event.xdata
                self.y1 = event.ydata
                self.selectrect.set_width(self.x1 - self.x0)
                self.selectrect.set_height(self.y1 - self.y0)
                self.selectrect.set_xy((self.x0, self.y0))
                self.ax.figure.canvas.draw()

    def readdata(self):
        self.data = np.random.rand(20, 20)


class DraggableRectangle(object):
    def __init__(self, artists, connectingline, arrows,
                 line_segments_vert=None,
                 line_segments_hor=None,
                 nbsteps_w_h=None,
                 texts=None,
                 tolerance=5, parent=None, framedim=(2048, 2048),
                 datatype='pixels'):
        for artist in artists:
            artist.set_picker(tolerance)
        self.artists = artists
        self.currently_dragging = False
        self.current_artist = None
        self.connectingline = connectingline
        self.arrows = arrows

        self.line_segments_vert = line_segments_vert
        self.line_segments_hor = line_segments_hor
        self.nbsteps_w_h = nbsteps_w_h

        self.texts = texts

        self.parent = parent
        self.framedim = framedim
        self.offset = (0, 0)

        self.datatype = datatype

        self.keyshiftpressed = False

        canvaslist = [artist.figure.canvas for artist in self.artists]
#         print "available canvas", canvaslist

        for canvas in set(canvaslist):
            canvas.mpl_connect('button_press_event', self.on_press)
            canvas.mpl_connect('button_release_event', self.on_release)
            canvas.mpl_connect('pick_event', self.on_pick)
            canvas.mpl_connect('motion_notify_event', self.on_motion)
            canvas.mpl_connect('key_press_event', self.on_key_press)

    def on_key_press(self, event):
        print("evt in drag points", event)

        key = event.key

        print("key", key)

        if key in ('shift',):
            self.keyshiftpressed = not self.keyshiftpressed

            return

        if key in ('up', 'down'):
            if key == 'down':
                self.nbsteps_w_h[0] -= 1

                if self.nbsteps_w_h[0] <= 2:
                    self.nbsteps_w_h[0] = 2

            elif key == 'up':
                self.nbsteps_w_h[0] += 1

            # gridlines management
            pt_lb = self.artists[0].center
            pt_lt = self.artists[1].center
            pt_rb = self.artists[3].center
            nbsteps_w_h = self.nbsteps_w_h
            segv, _segh = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, self.nbsteps_w_h[0], self.nbsteps_w_h[1])

            self.line_segments_vert.set_verts(segv)
            self.line_segments_hor.set_verts(_segh)

            # update press pt  and plot
            self.press = event.xdata, event.ydata

            self.connectingline.figure.canvas.draw()


            return


        if key in ('left', 'right'):
            if key == 'left':
                self.nbsteps_w_h[1] -= 1

                if self.nbsteps_w_h[1] <= 2:
                    self.nbsteps_w_h[1] = 2

            elif key == 'right':
                self.nbsteps_w_h[1] += 1

            # gridlines management
            pt_lb = self.artists[0].center
            pt_lt = self.artists[1].center
            pt_rb = self.artists[3].center
            nbsteps_w_h = self.nbsteps_w_h
            segv, _segh = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, self.nbsteps_w_h[0], self.nbsteps_w_h[1])

            self.line_segments_vert.set_verts(segv)
            self.line_segments_hor.set_verts(_segh)

            # update press pt  and plot
            self.press = event.xdata, event.ydata

            self.connectingline.figure.canvas.draw()


            return

        if key in ('p', 'm'):

            if key == 'p':
                self.increaseradius = 1
            elif key == 'm':
                self.increaseradius = -1

            self.changecircleradius()

        else:
            return

    def on_press(self, event):

        # don t mix pan/zoom with right click dragging
        toolbar = self.connectingline.figure.canvas.toolbar
        if toolbar.mode != '':
            return

        self.currently_dragging = True

        if event.button not in (1, '1',):
            print("pressing button %d" % event.button)
            self.press = event.xdata, event.ydata

            if not self.keyshiftpressed:
                self.rotatingrectangle = True
            else:
                self.rotatingrectangle = False

        else:
            self.rotatingrectangle = False

    def on_release(self, event):
        self.currently_dragging = False

        if event.button not in (1, '1',):
            self.rotatingrectangle = False
        else:
            self.current_artist = None

    def on_pick(self, event):
#         print "picking in dragpoint"
        if self.current_artist is None:
            self.current_artist = event.artist
            if event.artist != self.connectingline:
                print("touching a circle at")
                x0, y0 = event.artist.center
                print('center=', x0, y0)
                x1, y1 = event.mouseevent.xdata, event.mouseevent.ydata
                self.offset = (x0 - x1), (y0 - y1)

                print("self.offset", self.offset)
                return True

    def on_motion(self, event):
#         print "I m dragging in dragpoint!"
        if not self.currently_dragging:
            return
        if self.current_artist is None and not self.rotatingrectangle and not self.keyshiftpressed:
            return

        if self.datatype == 'pixels':
            limited_to_positive_integers = True
            xylimits = None
        elif self.datatype == 'gnomon':
            limited_to_positive_integers = False
            xylimits = (-1, 1, -1, 1)
        elif self.datatype == '2thetachi':
            # draw lines is useless
            return

#         print 'self.offset in onMotion', self.offset

        # handle rectangle rotation by moving right btn of mouse
        if event.button not in (1, '1',):

            if not self.keyshiftpressed:
                xpress, ypress = self.press
                print(" self.press", self.press)
                dx = event.xdata - xpress
                dy = event.ydata - ypress

                print("dx,dy", dx, dy)

                xMnew, yMnew = event.xdata, event.ydata
                xM, yM = xpress, ypress

                xs, ys = self.get_listpts_line()

                xC, yC = center_pts((xs[0], ys[0]), (xs[2], ys[2]))

                print("xC,yC", xC, yC)

                norme_dCM = 1.0 * math.sqrt((xC - xM) ** 2 + (yC - yM) ** 2)
                norme_dCMnew = 1.0 * math.sqrt((xC - xMnew) ** 2 + (yC - yMnew) ** 2)

                angle = math.acos(((xMnew - xC) * (xM - xC) + (yMnew - yC) * (yM - yC)) / norme_dCM / norme_dCMnew)
                cosangle = math.cos(angle)
                sinangle = math.sqrt(1 - cosangle ** 2)

                scal = (-dx * (yMnew - yC) + dy * (xMnew - xC))
                if scal < 0:
                    sign_angle = -1
                else:
                    sign_angle = 1

                MatRot = np.array([[cosangle, -sinangle * sign_angle],
                                   [sinangle * sign_angle, cosangle]])

                # last elem is the same that the fi#             angle = math.atan(0.04 * ((-dx * (yMnew - yC) + dy * (xMnew - xC)) / norme_dperpCM / norme_dCM))
    #             sinangle = math.sin(angle)
    #             cosangle = math.sqrt(1 - sinangle ** 2)rst to draw a line
                x_sc = np.array(xs[:-1]) - xC
                y_sc = np.array(ys[:-1]) - yC

                Corners = (np.array([x_sc, y_sc]).T)

                print("Corners", Corners)

                XCorners, YCorners = np.dot(MatRot, Corners.T)

                xs[:-1] = XCorners + xC
                ys[:-1] = YCorners + yC
                xs[-1] = xs[0]
                ys[-1] = ys[0]

                self.artists[0].center = xs[0], ys[0]
                self.artists[1].center = xs[1], ys[1]
                self.artists[2].center = xs[2], ys[2]
                self.artists[3].center = xs[3], ys[3]

                # arrows management
                xybase, xytip = self.getnewarrowparameters(xs, ys)
#                 print dir(self.arrows[0])

                self.arrows[0].xytip = xytip[0]
                self.arrows[1].xytip = xytip[1]
                self.arrows[0].xybase = xybase[0]
                self.arrows[1].xybase = xybase[1]

                # gridlines management
                pt_lb = self.artists[0].center
                pt_lt = self.artists[1].center
                pt_rb = self.artists[3].center
                nbsteps_w_h = self.nbsteps_w_h
                segv, _segh = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, nbsteps_w_h[0], nbsteps_w_h[1])

                self.line_segments_vert.set_verts(segv)
                self.line_segments_hor.set_verts(_segh)


                # manage dimension texts
                pt_rt = self.artists[2].center
                posw, wlen, posh, hlen = get_dimensionstext(pt_lb, pt_lt, pt_rt, pt_rb)

#                 print "sedgfsd", dir(self.texts[0])
                self.texts[0].set_x(posw[0])
                self.texts[0].set_y(posw[1])

                self.texts[0].set_text('%.2f microns' % wlen)

                self.texts[1].set_x(posh[0])
                self.texts[1].set_y(posh[1])

                self.texts[1].set_text('%.2f microns' % hlen)

                # update press pt  and plot
                self.press = event.xdata, event.ydata

                self.connectingline.set_data(xs, ys)
                self.connectingline.figure.canvas.draw()

                return
            # handle rectangle size  by moving right btn of mouse
            else:
                xs, ys = self.get_listpts_line()

                h_vector = np.array([xs[1] - xs[0], ys[1] - ys[0]])
                w_vector = np.array([xs[3] - xs[0], ys[3] - ys[0]])

                wnorme = np.sqrt(w_vector[0] ** 2 + w_vector[1] ** 2)
                hnorme = np.sqrt(h_vector[0] ** 2 + h_vector[1] ** 2)

                xpress, ypress = self.press
                print(" self.press", self.press)
                dx = event.xdata - xpress
                dy = event.ydata - ypress

                print("dx,dy", dx, dy)
                vecMMprime = [dx, dy]

                vec_dw = np.dot(vecMMprime, w_vector) * w_vector / wnorme ** 2
                vec_dh = np.dot(vecMMprime, h_vector) * h_vector / hnorme ** 2

                xcorners, ycorners = xs[:-1], ys[:-1]

                corners = np.array([xcorners, ycorners]).T

                corners[0] = corners[0] - vec_dw - vec_dh
                corners[1] = corners[1] - vec_dw + vec_dh
                corners[2] = corners[2] + vec_dw + vec_dh
                corners[3] = corners[3] + vec_dw - vec_dh

                for k in range(4):
                    xs[k], ys[k] = corners[k]
                xs[4], ys[4] = xs[0], ys[0]

                self.artists[0].center = xs[0], ys[0]
                self.artists[1].center = xs[1], ys[1]
                self.artists[2].center = xs[2], ys[2]
                self.artists[3].center = xs[3], ys[3]

                self.press = event.xdata, event.ydata

                # arrows management
                xybase, xytip = self.getnewarrowparameters(xs, ys)
#                 print dir(self.arrows[0])

                self.arrows[0].xytip = xytip[0]
                self.arrows[1].xytip = xytip[1]
                self.arrows[0].xybase = xybase[0]
                self.arrows[1].xybase = xybase[1]

                # gridlines management
                pt_lb = self.artists[0].center
                pt_lt = self.artists[1].center
                pt_rb = self.artists[3].center
                nbsteps_w_h = self.nbsteps_w_h
                segv, _segh = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, nbsteps_w_h[0], nbsteps_w_h[1])

                self.line_segments_vert.set_verts(segv)
                self.line_segments_hor.set_verts(_segh)

                # manage dimension texts
                pt_rt = self.artists[2].center
                posw, wlen, posh, hlen = get_dimensionstext(pt_lb, pt_lt, pt_rt, pt_rb)

#                 print "sedgfsd", dir(self.texts[0])
                self.texts[0].set_x(posw[0])
                self.texts[0].set_y(posw[1])

                self.texts[0].set_text('%.2f microns' % wlen)

                self.texts[1].set_x(posh[0])
                self.texts[1].set_y(posh[1])

                self.texts[1].set_text('%.2f microns' % hlen)

                # update press pt  and plot
                self.press = event.xdata, event.ydata


                self.connectingline.set_data(xs, ys)
                self.connectingline.figure.canvas.draw()

                return

        # hangle reaction of rectangle by moving either center, or one the 4 corners
        dx, dy = self.offset

        # central point
        if self.current_artist == self.artists[4]:

            if event.xdata is None:
                X4 = self.artists[4].center[0]
            else:
                X4 = event.xdata + dx
            if event.ydata is None:
                Y4 = self.artists[4].center[1]
            else:
                Y4 = event.ydata + dy

            evtx, evty = keep_in_imagearray(X4, Y4, self.framedim,
                                            limited_to_positive_integers=limited_to_positive_integers,
                                            xylimits=xylimits
                                            )

            self.artists[4].center = evtx, evty
#             self.current_artist.figure.canvas.draw()

#             print 'moving pt 0'
            xs, ys = self.get_listpts_line()
            print("xs, ys", xs, ys)
#             print "xs0, ys0", xs[0], ys[0]

            h_vector = (xs[1] - xs[0], ys[1] - ys[0])
            w_vector = (xs[3] - xs[0], ys[3] - ys[0])

            print("h_vector", h_vector)
            print("w_vector", w_vector)

            if event is not None:
                # left click
                if event.button in (1, '1',):
                    ChangeLineLength = False
                else:
                    print("ChangeLineLength is True")
                    ChangeLineLength = True

            xcenter = self.artists[4].center[0]
            ycenter = self.artists[4].center[1]

            if not ChangeLineLength:
    #             print "xs0, ys0", xs[0], ys[0]
                xs[0], ys[0] = xcenter - h_vector[0] / 2. - w_vector[0] / 2., ycenter - h_vector[1] / 2. - w_vector[1] / 2.
                xs[1], ys[1] = xcenter - h_vector[0] / 2. + w_vector[0] / 2., ycenter - h_vector[1] / 2. + w_vector[1] / 2.
                xs[2], ys[2] = xcenter + h_vector[0] / 2. + w_vector[0] / 2., ycenter + h_vector[1] / 2. + w_vector[1] / 2.
                xs[3], ys[3] = xcenter + h_vector[0] / 2. - w_vector[0] / 2., ycenter + h_vector[1] / 2. - w_vector[1] / 2.

                xs[4] = xs[0]
                ys[4] = ys[0]

                self.artists[0].center = xs[0], ys[0]
                self.artists[1].center = xs[1], ys[1]
                self.artists[2].center = xs[2], ys[2]
                self.artists[3].center = xs[3], ys[3]

                # arrows management
                xybase, xytip = self.getnewarrowparameters(xs, ys)
#                 print dir(self.arrows[0])

                self.arrows[0].xytip = xytip[0]
                self.arrows[1].xytip = xytip[1]
                self.arrows[0].xybase = xybase[0]
                self.arrows[1].xybase = xybase[1]

                # gridlines management
                pt_lb = self.artists[0].center
                pt_lt = self.artists[1].center
                pt_rb = self.artists[3].center
                nbsteps_w_h = self.nbsteps_w_h
                segv, _segh = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, nbsteps_w_h[0], nbsteps_w_h[1])

                self.line_segments_vert.set_verts(segv)
                self.line_segments_hor.set_verts(_segh)

                # manage dimension texts
                pt_rt = self.artists[2].center
                posw, wlen, posh, hlen = get_dimensionstext(pt_lb, pt_lt, pt_rt, pt_rb)

#                 print "sedgfsd", dir(self.texts[0])
                self.texts[0].set_x(posw[0])
                self.texts[0].set_y(posw[1])

                self.texts[0].set_text('%.2f microns' % wlen)

                self.texts[1].set_x(posh[0])
                self.texts[1].set_y(posh[1])

                self.texts[1].set_text('%.2f microns' % hlen)


#
            self.connectingline.set_data(xs, ys)
            self.connectingline.figure.canvas.draw()


        # first  point  left bottom xs[0], ys[0]
        elif self.current_artist == self.artists[0]:

            if event.xdata is None:
                X0 = self.artists[0].center[0]
            else:
                X0 = event.xdata + dx
            if event.ydata is None:
                Y0 = self.artists[0].center[1]
            else:
                Y0 = event.ydata + dy

            evtx, evty = keep_in_imagearray(X0, Y0, self.framedim,
                                            limited_to_positive_integers=limited_to_positive_integers,
                                            xylimits=xylimits
                                            )

            self.artists[0].center = evtx, evty
#             self.current_artist.figure.canvas.draw()

#             print 'moving pt 0'
            xs, ys = self.get_listpts_line()
#             print "xs, ys", xs, ys
#             print "xs0, ys0", xs[0], ys[0]

            if event is not None:
                # left click
                if event.button in (1, '1',):
                    ChangeLineLength = False
                else:
                    ChangeLineLength = True

            xs[0] = self.artists[0].center[0]
            ys[0] = self.artists[0].center[1]

            # xs[2] fixed
            if not ChangeLineLength:
    #             print "xs0, ys0", xs[0], ys[0]

                # old frame  : h = 01 or 32  w = 12 or 03  m 02/2
                h_vector = (xs[2] - xs[3], ys[2] - ys[3])
                w_vector = (xs[2] - xs[1], ys[2] - ys[1])

#                 old_O = pt2 - (h_vector+w_vector)

                xO, yO = xs[2] - (h_vector[0] + w_vector[0]), ys[2] - (h_vector[1] + w_vector[1])

                x_newcenter, y_newcenter = center_pts((xs[0], ys[0]), (xs[2], ys[2]))

                # vec 0'1' = h_vector(1-(OO'.h_vector)/h_vector**2) = h_vector*fac1
                # vec 0'3' = w_vector(1-(OO'.w_vector)/w_vector**2) = w_vector*fac2

                vecOOprime = [xs[0] - xO, ys[0] - yO]

                fac1 = 1 - np.dot(vecOOprime, h_vector) / np.dot(h_vector, h_vector)
                fac2 = 1 - np.dot(vecOOprime, w_vector) / np.dot(w_vector, w_vector)

                xs[1] = xs[0] + fac1 * h_vector[0]
                ys[1] = ys[0] + fac1 * h_vector[1]

                xs[3] = xs[0] + fac2 * w_vector[0]
                ys[3] = ys[0] + fac2 * w_vector[1]

                xs[4] = xs[0]
                ys[4] = ys[0]


                self.artists[4].center = x_newcenter, y_newcenter
                self.artists[1].center = xs[1], ys[1]
#                 self.artists[2].center = xs[2], ys[2]
                self.artists[3].center = xs[3], ys[3]
#             # other  pt 0  move effect
#             else:
#                 # TODO
#                 xs[2] = xs[0] + 2 * (xs[1] - xs[0])
#                 ys[2] = ys[0] + 2 * (ys[1] - ys[0])
#
#                 self.artists[2].center = xs[2], ys[2]

                # arrows management
                xybase, xytip = self.getnewarrowparameters(xs, ys)
#                 print dir(self.arrows[0])

                self.arrows[0].xytip = xytip[0]
                self.arrows[1].xytip = xytip[1]
                self.arrows[0].xybase = xybase[0]
                self.arrows[1].xybase = xybase[1]

                # gridlines management
                pt_lb = self.artists[0].center
                pt_lt = self.artists[1].center
                pt_rb = self.artists[3].center
                nbsteps_w_h = self.nbsteps_w_h
                segv, _segh = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, nbsteps_w_h[0], nbsteps_w_h[1])

                self.line_segments_vert.set_verts(segv)
                self.line_segments_hor.set_verts(_segh)

                # manage dimension texts
                pt_rt = self.artists[2].center
                posw, wlen, posh, hlen = get_dimensionstext(pt_lb, pt_lt, pt_rt, pt_rb)

#                 print "sedgfsd", dir(self.texts[0])
                self.texts[0].set_x(posw[0])
                self.texts[0].set_y(posw[1])

                self.texts[0].set_text('%.2f microns' % wlen)

                self.texts[1].set_x(posh[0])
                self.texts[1].set_y(posh[1])

                self.texts[1].set_text('%.2f microns' % hlen)


            self.connectingline.set_data(xs, ys)
            self.connectingline.figure.canvas.draw()

        # 2nd  point  left top xs[1], ys[1]
        elif self.current_artist == self.artists[1]:
#             print "moving self.artists[1]"
            if event.xdata is None:
                X1 = self.artists[1].center[0]
            else:
                X1 = event.xdata + dx
            if event.ydata is None:
                Y1 = self.artists[1].center[1]
            else:
                Y1 = event.ydata + dy

            evtx, evty = keep_in_imagearray(X1, Y1, self.framedim,
                                            limited_to_positive_integers=limited_to_positive_integers,
                                            xylimits=xylimits
                                            )

            self.artists[1].center = evtx, evty
#             self.current_artist.figure.canvas.draw()

#             print 'moving pt 0'
            xs, ys = self.get_listpts_line()
#             print "xs, ys", xs, ys
#             print "xs0, ys0", xs[0], ys[0]

            if event is not None:
                # left click
                if event.button in (1, '1',):
                    ChangeLineLength = False
                else:
                    ChangeLineLength = True

            xs[1] = self.artists[1].center[0]
            ys[1] = self.artists[1].center[1]

            # xs[3] fixed
            if not ChangeLineLength:
    #             print "xs0, ys0", xs[0], ys[0]

                # old frame  : h = 01 or 32  w = 12 or 03  m 02/2
                h_vector = (xs[2] - xs[3], ys[2] - ys[3])
                w_vector = (xs[3] - xs[0], ys[3] - ys[0])

#                 old_1 = pt3 + h_vector-w_vector

                x1, y1 = xs[3] - w_vector[0] + h_vector[0], ys[3] - w_vector[1] + h_vector[1]

                x_newcenter, y_newcenter = center_pts((xs[1], ys[1]), (xs[3], ys[3]))

                # vec 1'0' = -h_vector(1+(11'.h_vector)/h_vector**2) = h_vector*fac1
                # vec 1'2' = w_vector(1-(11'.w_vector)/w_vector**2) = w_vector*fac2

                vec11prime = [xs[1] - x1, ys[1] - y1]

                fac1 = -(1 + np.dot(vec11prime, h_vector) / np.dot(h_vector, h_vector))
                fac2 = 1 - np.dot(vec11prime, w_vector) / np.dot(w_vector, w_vector)

                xs[0] = xs[1] + fac1 * h_vector[0]
                ys[0] = ys[1] + fac1 * h_vector[1]

                xs[2] = xs[1] + fac2 * w_vector[0]
                ys[2] = ys[1] + fac2 * w_vector[1]

                xs[4] = xs[0]
                ys[4] = ys[0]

                self.artists[4].center = x_newcenter, y_newcenter
                self.artists[0].center = xs[0], ys[0]
#                 self.artists[2].center = xs[2], ys[2]
                self.artists[2].center = xs[2], ys[2]

#             # other  pt 0  move effect
#             else:
#                 # TODO
#                 xs[2] = xs[0] + 2 * (xs[1] - xs[0])
#                 ys[2] = ys[0] + 2 * (ys[1] - ys[0])
#
#                 self.artists[2].center = xs[2], ys[2]
                # arrows management
                xybase, xytip = self.getnewarrowparameters(xs, ys)
#                 print dir(self.arrows[0])

                self.arrows[0].xytip = xytip[0]
                self.arrows[1].xytip = xytip[1]
                self.arrows[0].xybase = xybase[0]
                self.arrows[1].xybase = xybase[1]

                # gridlines management
                pt_lb = self.artists[0].center
                pt_lt = self.artists[1].center
                pt_rb = self.artists[3].center
                nbsteps_w_h = self.nbsteps_w_h
                segv, _segh = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, nbsteps_w_h[0], nbsteps_w_h[1])

                self.line_segments_vert.set_verts(segv)
                self.line_segments_hor.set_verts(_segh)

                # manage dimension texts
                pt_rt = self.artists[2].center
                posw, wlen, posh, hlen = get_dimensionstext(pt_lb, pt_lt, pt_rt, pt_rb)

#                 print "sedgfsd", dir(self.texts[0])
                self.texts[0].set_x(posw[0])
                self.texts[0].set_y(posw[1])

                self.texts[0].set_text('%.2f microns' % wlen)

                self.texts[1].set_x(posh[0])
                self.texts[1].set_y(posh[1])

                self.texts[1].set_text('%.2f microns' % hlen)


            self.connectingline.set_data(xs, ys)
            self.connectingline.figure.canvas.draw()

        # 3rd  point  right top xs[2], ys[2]
        elif self.current_artist == self.artists[2]:
#             print "moving self.artists[2]"

            if event.xdata is None:
                X2 = self.artists[2].center[0]
            else:
                X2 = event.xdata + dx
            if event.ydata is None:
                Y2 = self.artists[2].center[1]
            else:
                Y2 = event.ydata + dy

            evtx, evty = keep_in_imagearray(X2, Y2, self.framedim,
                                            limited_to_positive_integers=limited_to_positive_integers,
                                            xylimits=xylimits
                                            )

            self.artists[2].center = evtx, evty
#             self.current_artist.figure.canvas.draw()

#             print 'moving pt 0'
            xs, ys = self.get_listpts_line()
#             print "xs, ys", xs, ys
#             print "xs0, ys0", xs[0], ys[0]

            if event is not None:
                # left click
                if event.button in (1, '1',):
                    ChangeLineLength = False
                else:
                    ChangeLineLength = True

            xs[2] = self.artists[2].center[0]
            ys[2] = self.artists[2].center[1]

            # xs[0] fixed
            if not ChangeLineLength:
    #             print "xs0, ys0", xs[0], ys[0]

                # old frame  : h = 01 or 32  w = 12 or 03  m 02/2
                h_vector = (xs[1] - xs[0], ys[1] - ys[0])
                w_vector = (xs[3] - xs[0], ys[3] - ys[0])

#                 old_2 = pt0 + h_vector+w_vector

                x2, y2 = xs[0] + w_vector[0] + h_vector[0], ys[0] + w_vector[1] + h_vector[1]

                x_newcenter, y_newcenter = center_pts((xs[0], ys[0]), (xs[2], ys[2]))

                # vec 2'3' = -h_vector(1+(22'.h_vector)/h_vector**2) = h_vector*fac1
                # vec 2'1' = -w_vector(1+(22'.w_vector)/w_vector**2) = w_vector*fac2

                vec22prime = [xs[2] - x2, ys[2] - y2]
                print("vec22prime", vec22prime)

                fac1 = -(1 + np.dot(vec22prime, h_vector) / np.dot(h_vector, h_vector))
                fac2 = -(1 + np.dot(vec22prime, w_vector) / np.dot(w_vector, w_vector))

                print("fac1 and fac2", fac1, fac2)

                xs[3] = xs[2] + fac1 * h_vector[0]
                ys[3] = ys[2] + fac1 * h_vector[1]

                xs[1] = xs[2] + fac2 * w_vector[0]
                ys[1] = ys[2] + fac2 * w_vector[1]

                xs[4] = xs[0]
                ys[4] = ys[0]

                self.artists[4].center = x_newcenter, y_newcenter
                self.artists[3].center = xs[3], ys[3]
#                 self.artists[2].center = xs[2], ys[2]
                self.artists[1].center = xs[1], ys[1]

                # arrows management
                xybase, xytip = self.getnewarrowparameters(xs, ys)
#                 print dir(self.arrows[0])

                self.arrows[0].xytip = xytip[0]
                self.arrows[1].xytip = xytip[1]
                self.arrows[0].xybase = xybase[0]
                self.arrows[1].xybase = xybase[1]

                # gridlines management
                pt_lb = self.artists[0].center
                pt_lt = self.artists[1].center
                pt_rb = self.artists[3].center
                nbsteps_w_h = self.nbsteps_w_h
                segv, _segh = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, nbsteps_w_h[0], nbsteps_w_h[1])

                self.line_segments_vert.set_verts(segv)
                self.line_segments_hor.set_verts(_segh)

                # manage dimension texts
                pt_rt = self.artists[2].center
                posw, wlen, posh, hlen = get_dimensionstext(pt_lb, pt_lt, pt_rt, pt_rb)

#                 print "sedgfsd", dir(self.texts[0])
                self.texts[0].set_x(posw[0])
                self.texts[0].set_y(posw[1])

                self.texts[0].set_text('%.2f microns' % wlen)

                self.texts[1].set_x(posh[0])
                self.texts[1].set_y(posh[1])

                self.texts[1].set_text('%.2f microns' % hlen)

            self.connectingline.set_data(xs, ys)
            self.connectingline.figure.canvas.draw()

        # 4th  point  right bottom xs[3], ys[3]
        elif self.current_artist == self.artists[3]:

            if event.xdata is None:
                X3 = self.artists[3].center[0]
            else:
                X3 = event.xdata + dx
            if event.ydata is None:
                Y3 = self.artists[3].center[1]
            else:
                Y3 = event.ydata + dy

            evtx, evty = keep_in_imagearray(X3, Y3, self.framedim,
                                            limited_to_positive_integers=limited_to_positive_integers,
                                            xylimits=xylimits
                                            )

            self.artists[3].center = evtx, evty
#             self.current_artist.figure.canvas.draw()

#             print 'moving pt 0'
            xs, ys = self.get_listpts_line()
#             print "xs, ys", xs, ys
#             print "xs0, ys0", xs[0], ys[0]

            if event is not None:
                # left click
                if event.button in (1, '1',):
                    ChangeLineLength = False
                else:
                    ChangeLineLength = True

            xs[3] = self.artists[3].center[0]
            ys[3] = self.artists[3].center[1]

            # xs[1] fixed
            if not ChangeLineLength:

                # old frame  : h = 01 or 32  w = 12 or 03  m 02/2
                h_vector = (xs[1] - xs[0], ys[1] - ys[0])
                w_vector = (xs[2] - xs[1], ys[2] - ys[1])

#                 old_3 = pt1 - h_vector+w_vector

                x3, y3 = xs[1] + w_vector[0] - h_vector[0], ys[1] + w_vector[1] - h_vector[1]

                x_newcenter, y_newcenter = center_pts((xs[3], ys[3]), (xs[1], ys[1]))

                # vec 3'2' = h_vector(1-(33'.h_vector)/h_vector**2) = h_vector*fac1
                # vec 3'0' = -w_vector(1+(33'.w_vector)/w_vector**2) = w_vector*fac2

                vec33prime = [xs[3] - x3, ys[3] - y3]

                fac1 = 1 - np.dot(vec33prime, h_vector) / np.dot(h_vector, h_vector)
                fac2 = -(1 + np.dot(vec33prime, w_vector) / np.dot(w_vector, w_vector))

                xs[2] = xs[3] + fac1 * h_vector[0]
                ys[2] = ys[3] + fac1 * h_vector[1]

                xs[0] = xs[3] + fac2 * w_vector[0]
                ys[0] = ys[3] + fac2 * w_vector[1]

                xs[4] = xs[0]
                ys[4] = ys[0]

                self.artists[4].center = x_newcenter, y_newcenter
                self.artists[2].center = xs[2], ys[2]
                self.artists[0].center = xs[0], ys[0]

                # arrows management
                xybase, xytip = self.getnewarrowparameters(xs, ys)
#                 print dir(self.arrows[0])

                self.arrows[0].xytip = xytip[0]
                self.arrows[1].xytip = xytip[1]
                self.arrows[0].xybase = xybase[0]
                self.arrows[1].xybase = xybase[1]

                # gridlines management
                pt_lb = self.artists[0].center
                pt_lt = self.artists[1].center
                pt_rb = self.artists[3].center
                nbsteps_w_h = self.nbsteps_w_h
                segv, _segh = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, nbsteps_w_h[0], nbsteps_w_h[1])

                self.line_segments_vert.set_verts(segv)
                self.line_segments_hor.set_verts(_segh)

#             # other  pt 0  move effect
#             else:
#                 # TODO
#                 xs[2] = xs[0] + 2 * (xs[1] - xs[0])
#                 ys[2] = ys[0] + 2 * (ys[1] - ys[0])
#
#                 self.artists[2].center = xs[2], ys[2]

            self.connectingline.set_data(xs, ys)
            self.connectingline.figure.canvas.draw()


        if self.parent is not None:
            # TODO to replace by  self.parent.viewingLUTpanel.myfunction(local parameters: artist center)
#             print "self.parent in DraggableLine", self.parent
            try:
                self.parent.viewingLUTpanel.x0, self.parent.viewingLUTpanel.y0 = self.artists[0].center
                self.parent.viewingLUTpanel.x2, self.parent.viewingLUTpanel.y2 = self.artists[2].center
                self.parent.viewingLUTpanel.updateLineProfile()
            except AttributeError:
                return

    def getnewarrowparameters(self, xs, ys):
        """
        from xs 's positions (line points)
        """
        xytip = [[xs[3], ys[3]], [xs[1], ys[1]]]
        xybase = [[xs[0], ys[0]], [xs[0], ys[0]]]
        return xybase, xytip

    def get_listpts_line(self):
        xs = np.array(list(self.connectingline.get_xdata()))
        ys = np.array(list(self.connectingline.get_ydata()))

        return xs, ys

    def changecircleradius(self):

        for artist in self.artists:
            artist.radius = int(artist.radius + self.increaseradius * 5.)

    def change_line_length(self):

        print("change_line_lengths")
        xs, ys = self.get_listpts_line()

        print("xs, ys", xs, ys)

        half_length_x = 0.5 * (xs[2] - xs[0])
        half_length_y = 0.5 * (ys[2] - ys[0])

        factor = 0.5

        if self.decrease_line_length == True:
            factor = .5
        elif self.increase_line_length == True:
            factor = 1.5

        xs[0] = xs[1] - int(half_length_x * factor)
        ys[0] = ys[1] - int(half_length_y * factor)

        xs[2] = xs[1] + int(half_length_x * factor)
        ys[2] = ys[1] + int(half_length_y * factor)

        self.artists[0].center = xs[0], ys[0]
        self.artists[2].center = xs[2], ys[2]

        self.decrease_line_length = False
        self.increase_line_length = False

        self.connectingline.set_data(xs, ys)
        self.connectingline.figure.canvas.draw()

def keep_in_imagearray(x, y, framedim,
                       limited_to_positive_integers=True, xylimits=None):

        # for image pixel data
        if limited_to_positive_integers:
            if x < 0: x = 0
            if x >= framedim[1]: x = framedim[1] - 1

            if y < 0: y = 0
            if y >= framedim[0]: y = framedim[0] - 1
        # for other float with float x y coordinates
        else:
            if xylimits is not None:
                xmin, xmax, ymin, ymax = xylimits
            else:
                xmin, xmax, ymin, ymax = -1., 1., -1., 1.

            epsilonshift = (xmax - xmin) / 200.

            if x < xmin:
                x = xmin + epsilonshift
            if x >= xmax:
                x = xmax - epsilonshift

            if y < ymin:
                y = ymin + epsilonshift
            if y >= ymax:
                y = ymax - epsilonshift

        return x, y

def center_pts(pt1, pt2):
    return [(pt1[0] + pt2[0]) / 2., (pt1[1] + pt2[1]) / 2.]


def getsegs_forlines_2(pt_lb, pt_rb, pt_lt, w_nbsteps, h_nbsteps):
    """
    return vertical and horizontal lines collections
    """
    w_vector = [pt_rb[0] - pt_lb[0], pt_rb[1] - pt_lb[1]]
    w_length = np.sqrt(np.dot(w_vector, w_vector))
    w_unit = w_vector / w_length

    w_steplength = 1.0 * w_length / w_nbsteps

    h_vector = [pt_lt[0] - pt_lb[0], pt_lt[1] - pt_lb[1]]
    h_length = np.sqrt(np.dot(h_vector, h_vector))
    h_unit = h_vector / h_length

    h_steplength = 1.0 * h_length / h_nbsteps

    print("w_length", w_length)
    print("h_length", h_length)

    print("w_steplength", w_steplength)
    print("h_steplength", h_steplength)

    nodes_uv = twoindices_positive_up_to(w_nbsteps, 1)

    NodesUV = nodes_uv.reshape((2, w_nbsteps + 1, 2)).transpose((1, 0, 2)).reshape((2 * (w_nbsteps + 1), 2))

    print("nodes_uv", NodesUV)

    nodes_xy = np.dot(NodesUV, np.array([w_unit * w_steplength, h_vector])) + pt_lb

    print("nodes_xy.shape", nodes_xy.shape)

    segs_vert = nodes_xy.reshape((w_nbsteps + 1, 2, 2))

    #  horizontal lines
    nodes_uv = twoindices_positive_up_to(h_nbsteps, 1)

    NodesUV = nodes_uv.reshape((2, h_nbsteps + 1, 2)).transpose((1, 0, 2)).reshape((2 * (h_nbsteps + 1), 2))

    print("nodes_uv", NodesUV)

    nodes_xy = np.dot(NodesUV, np.array([h_unit * h_steplength, w_vector])) + pt_lb

    print("nodes_xy.shape", nodes_xy.shape)

    segs_hor = nodes_xy.reshape((h_nbsteps + 1, 2, 2))

    return segs_vert, segs_hor

def get_dimensionstext(pt_lb, pt_lt, pt_rt, pt_rb):
    h_vector = np.array([pt_lt[0] - pt_lb[0], pt_lt[1] - pt_lb[1]])
    w_vector = np.array([pt_rb[0] - pt_lb[0], pt_rb[1] - pt_lb[1]])
    ptcenter = np.array([0.5 * (pt_lb[0] + pt_rt[0]),
                         0.5 * (pt_lb[1] + pt_rt[1])])

    text_w_pos = ptcenter - .85 * h_vector
    text_h_pos = ptcenter + .85 * w_vector
    h_length = np.sqrt(np.dot(h_vector, h_vector))
    w_length = np.sqrt(np.dot(w_vector, w_vector))



    return text_w_pos, w_length, text_h_pos, h_length



if __name__ == '__main__':
    fig, ax = plt.subplots()
    ax.set(xlim=[-1, 21], ylim=[-1, 21])

    pt_lb = [10., 10.]
    pt_lt = [10., 12.]
    pt_rt = [15., 12.]
    pt_rb = [15., 10.]

    from matplotlib.collections import LineCollection

    w_nbsteps = 3
    h_nbsteps = 5

    segs_vert, segs_hor = getsegs_forlines_2(pt_lb, pt_rb, pt_lt, w_nbsteps, h_nbsteps)
# 
    line_segments_vert = LineCollection(segs_vert,
                                linestyle='solid')
#     ax.add_collection(line_segments_vert)
# 
    line_segments_hor = LineCollection(segs_hor,
                                linestyle='solid', colors='r')
#     ax.add_collection(line_segments_hor)
# 
# 
    ptcenter = center_pts(pt_lb, pt_rt)
# 
    circles = [patches.Circle(pt_lb, 0.3, fc='g', alpha=0.5),
               patches.Circle(pt_lt, 0.3, fc='g', alpha=0.5),
               patches.Circle(pt_rt, 0.3, fc='g', alpha=0.5),
               patches.Circle(pt_rb, 0.3, fc='g', alpha=0.5),
               patches.Circle(ptcenter, 0.3, fc='r', alpha=0.5)]
# 
# #     arrows = [patches.Arrow(pt_lb[0], pt_lb[1], pt_lt[0] - pt_lb[0], pt_lt[1] - pt_lb[1], fc='g', width=2.0, alpha=0.5),
# #                patches.Arrow(pt_lb[0], pt_lb[1], pt_rb[0] - pt_lb[0], pt_rb[1] - pt_lb[1], fc='g', width=0.5, alpha=0.5)]
# 
#   # TODO use FancyArrow see SimulFrame.py
    arrows = [patches.YAArrow(fig, (pt_rb[0], pt_rb[1]), (pt_lb[0], pt_lb[1]),
                              fc='b', width=0.1, headwidth=.3, linestyle='dashed', alpha=0.5),
               patches.YAArrow(fig, (pt_lt[0], pt_lt[1]), (pt_lb[0], pt_lb[1]),
                               fc='b', width=0.3, headwidth=.9, alpha=0.5)]
# 
    drectangle, = ax.plot([pt_lb[0], pt_lt[0], pt_rt[0], pt_rb[0], pt_lb[0]],
                          [pt_lb[1], pt_lt[1], pt_rt[1], pt_rb[1], pt_lb[1]], picker=1.5)
# #     linebuilder = LineBuilder(line)
# 
    for circ in circles:
        ax.add_patch(circ)
    #
    for ar in arrows:
        ax.add_patch(ar)
# 
    text_w_pos, w_length, text_h_pos, h_length = get_dimensionstext(pt_lb, pt_lt, pt_rt, pt_rb)
# 
    textw = ax.text(text_w_pos[0], text_w_pos[1], '%.2f microns' % w_length)
    texth = ax.text(text_h_pos[0], text_h_pos[1], '%.2f microns' % h_length)
# 
# 
#     print "ax.patches", ax.patches

    dr = DraggableRectangle(circles, drectangle, arrows,
                            line_segments_vert=line_segments_vert,
                            line_segments_hor=line_segments_hor,
                            nbsteps_w_h=[w_nbsteps, h_nbsteps],
                            texts=[textw, texth],
                            tolerance=2.)
     


    plt.show()
#     a = Annotate()




import os
import sys
import re

import numpy as np

import wx

if wx.__version__ < "4.":
    WXPYTHON4 = False
else:
    WXPYTHON4 = True
    wx.OPEN = wx.FD_OPEN

if sys.version_info.major == 3:
    from .. import generaltools as GT
else:
    import generaltools as GT
    
DEG = np.pi/180.
# --- ---------------  Matrix Editor
class MatrixEditor_Dialog(wx.Frame):
    """
    class to handle edition of matrices

    parent lust have:
    dict_Rot
    """
    def __init__(self, parent, _id, title):

        wx.Frame.__init__(self, parent, _id, title, size=(600, 700))

        self.dirname = os.getcwd()

        self.parent = parent
        # variables
        self.modify = False
        self.last_name_saved = ""
        self.last_name_stored = ""
        self.replace = False

        font3 = wx.Font(10, wx.MODERN, wx.NORMAL, wx.BOLD)

        panel = wx.Panel(self, -1, style=wx.SIMPLE_BORDER, size=(590, 690), pos=(5, 5))

        self.rbeditor = wx.RadioButton(panel, -1, "Text Editor Input", (25, 10),
                                                                                style=wx.RB_GROUP)
        wx.StaticText(panel, -1, "[[#,#,#],[#,#,#],[#,#,#]]", (40, 25))

        self.text = wx.TextCtrl(panel, -1, "", pos=(50, 45), size=(250, 85),
                                                    style=wx.TE_MULTILINE | wx.TE_PROCESS_ENTER)
        self.text.SetFocus()
        self.text.Bind(wx.EVT_TEXT, self.OnTextChanged)
        self.text.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)

        self.rbelem = wx.RadioButton(panel, -1, "Matrix Elements Input", (25, 150))
        self.rbeditor.SetValue(True)
        self.mat_a11 = wx.TextCtrl(panel, -1, "1", (20, 180), size=(80, -1))
        self.mat_a12 = wx.TextCtrl(panel, -1, "0", (120, 180), size=(80, -1))
        self.mat_a13 = wx.TextCtrl(panel, -1, "0", (220, 180), size=(80, -1))
        self.mat_a21 = wx.TextCtrl(panel, -1, "0", (20, 210), size=(80, -1))
        self.mat_a22 = wx.TextCtrl(panel, -1, "1", (120, 210), size=(80, -1))
        self.mat_a23 = wx.TextCtrl(panel, -1, "0", (220, 210), size=(80, -1))
        self.mat_a31 = wx.TextCtrl(panel, -1, "0", (20, 240), size=(80, -1))
        self.mat_a32 = wx.TextCtrl(panel, -1, "0", (120, 240), size=(80, -1))
        self.mat_a33 = wx.TextCtrl(panel, -1, "1", (220, 240), size=(80, -1))

        self.CurrentMat = "Identity"
        List_Rot_name = list(self.parent.dict_Rot.keys())
        List_Rot_name.remove("Identity")
        List_Rot_name.sort()
        self.list_of_Rot = ["Identity"] + List_Rot_name

        rr = wx.StaticText(panel, -1, "Rotation", (400, 15))
        rr.SetFont(font3)
        wx.StaticText(panel, -1, "Axis angles(long., lat) or vector", (320, 45))
        self.longitude = wx.TextCtrl(panel, -1, "0.0", (330, 70), size=(60, -1))
        self.latitude = wx.TextCtrl(panel, -1, "0.0", (410, 70), size=(60, -1))
        wx.StaticText(panel, -1, "in deg.", (485, 75))
        self.axisrot = wx.TextCtrl(panel, -1, "[1, 0,0]", (390, 100), size=(60, -1))
        wx.StaticText(panel, -1, "Rotation angle", (370, 130))
        self.anglerot = wx.TextCtrl(panel, -1, "0.0", (390, 150), size=(60, -1))
        wx.StaticText(panel, -1, "in deg.", (485, 155))

        buttoncomputemat_1 = wx.Button(panel, 555, "Compute", pos=(330, 180), size=(80, 25))
        wx.StaticText(panel, -1, "from axis angles", (420, 185))
        buttoncomputemat_1.Bind(wx.EVT_BUTTON, self.OnComputeMatrix_axisangles, id=555)
        buttoncomputemat_2 = wx.Button(panel, 556, "Compute", pos=(330, 210), size=(80, 25))
        wx.StaticText(panel, -1, "from axis vector", (420, 215))
        buttoncomputemat_2.Bind(wx.EVT_BUTTON, self.OnComputeMatrix_axisvector, id=556)

        buttonread = wx.Button(panel, 101, "Look", pos=(20, 300), size=(60, 25))
        # buttonread.SetFont(font3)
        buttonread.Bind(wx.EVT_BUTTON, self.OnLookMatrix, id=101)
        self.comboRot = wx.ComboBox(panel, 6, "Identity", (100, 300), choices=self.list_of_Rot)
        self.comboRot.Bind(wx.EVT_COMBOBOX, self.EnterComboRot, id=6)

        buttonsave = wx.Button(panel, 102, "Save", pos=(20, 340), size=(60, 25))
        # buttonsave.SetFont(font3)
        buttonsave.Bind(wx.EVT_BUTTON, self.OnSaveFile, id=102)
        wx.StaticText(panel, -1, "in", (110, 345))
        self.filenamesave = wx.TextCtrl(panel, -1, "", (160, 340), size=(100, 25))
        wx.StaticText(panel, -1, "(on hard disk)", (260, 345))

        buttonstore = wx.Button(panel, 103, "Store", pos=(20, 380), size=(60, 25))
        # buttonstore.SetFont(font3)
        buttonstore.Bind(wx.EVT_BUTTON, self.OnStoreMatrix, id=103)
        wx.StaticText(panel, -1, "in", (110, 385))
        self.filenamestore = wx.TextCtrl(panel, -1, "", (160, 380), size=(100, 25))
        wx.StaticText(panel, -1, "(will appear in simulation matrix menu)", (260, 385))

        buttonload = wx.Button(panel, 104, "Load", pos=(20, 420), size=(60, 25))
        # buttonload.SetFont(font3)
        buttonload.Bind(wx.EVT_BUTTON, self.OnOpenMatrixFile, id=104)
        wx.StaticText(panel, -1,
                "Matrix from saved file in simple ASCII text editor format", (100, 425))

        buttonXMASload = wx.Button(panel, 105, "Read XMAS", pos=(20, 460), size=(100, 25))
        # buttonload.SetFont(font3)
        buttonXMASload.Bind(wx.EVT_BUTTON, self.OnLoadXMAS_INDfile, id=105)
        wx.StaticText(panel, -1, ".IND file", (130, 465))
        buttonXMASload.Disable()

        buttonconvert = wx.Button(panel, 106, "Convert", pos=(200, 460), size=(70, 25))
        # buttonload.SetFont(font3)
        buttonconvert.Bind(wx.EVT_BUTTON, self.OnConvert, id=106)
        wx.StaticText(panel, -1, "XMAS to LaueTools lab. frame", (290, 465))
        buttonconvert.Disable()

        buttonconvert2 = wx.Button(panel, 107, "Convert", pos=(200, 510), size=(70, 25))
        buttonconvert2.Bind(wx.EVT_BUTTON, self.OnConvertlabtosample, id=107)
        wx.StaticText(panel, -1, "LaueTools: from lab. to sample frame", (290, 515))
        buttonconvert2.Disable()

        self.invconvert = wx.CheckBox(panel, 108, "inv.", (210, 535))
        self.invconvert.SetValue(False)

        buttonquit = wx.Button(panel, 6, "Quit", (150, 620), size=(120, 50))
        buttonquit.Bind(wx.EVT_BUTTON, self.OnQuit, id=6)

        self.StatusBar()

        # tooltips
        buttonquit.SetToolTipString("Quit editor")

    def StatusBar(self):
        self.statusbar = self.CreateStatusBar()
        self.statusbar.SetFieldsCount(3)
        self.statusbar.SetStatusWidths([-5, -2, -1])

    def ToggleStatusBar(self, _):
        if self.statusbar.IsShown():
            self.statusbar.Hide()
        else:
            self.statusbar.Show()

    def OnTextChanged(self, evt):
        self.modify = True
        evt.Skip()

    def OnKeyDown(self, evt):
        keycode = evt.GetKeyCode()
        if keycode == wx.WXK_INSERT:
            if not self.replace:
                self.statusbar.SetStatusText("INS", 2)
                self.replace = True
            else:
                self.statusbar.SetStatusText("", 2)
                self.replace = False
        evt.Skip()

    def OnComputeMatrix_axisangles(self, _):
        longitude = float(self.longitude.GetValue())
        latitude = float(self.latitude.GetValue())
        angle = float(self.anglerot.GetValue())
        deg = DEG
        x = np.cos(longitude * deg) * np.cos(latitude * deg)
        y = np.sin(longitude * deg) * np.cos(latitude * deg)
        z = np.sin(latitude * deg)
        matrix = GT.matRot(np.array([x, y, z]), angle)

        self.mat_a11.SetValue(str(matrix[0][0]))
        self.mat_a12.SetValue(str(matrix[0][1]))
        self.mat_a13.SetValue(str(matrix[0][2]))
        self.mat_a21.SetValue(str(matrix[1][0]))
        self.mat_a22.SetValue(str(matrix[1][1]))
        self.mat_a23.SetValue(str(matrix[1][2]))
        self.mat_a31.SetValue(str(matrix[2][0]))
        self.mat_a32.SetValue(str(matrix[2][1]))
        self.mat_a33.SetValue(str(matrix[2][2]))
        self.text.SetValue(str(matrix.tolist()))

    def OnComputeMatrix_axisvector(self, _):
        Axis = str(self.axisrot.GetValue())
        angle = float(self.anglerot.GetValue())

        aa = Axis.split(",")
        x = float(aa[0][1:])
        y = float(aa[1])
        z = float(aa[2][:-1])
        matrix = GT.matRot(np.array([x, y, z]), angle)

        self.mat_a11.SetValue(str(matrix[0][0]))
        self.mat_a12.SetValue(str(matrix[0][1]))
        self.mat_a13.SetValue(str(matrix[0][2]))
        self.mat_a21.SetValue(str(matrix[1][0]))
        self.mat_a22.SetValue(str(matrix[1][1]))
        self.mat_a23.SetValue(str(matrix[1][2]))
        self.mat_a31.SetValue(str(matrix[2][0]))
        self.mat_a32.SetValue(str(matrix[2][1]))
        self.mat_a33.SetValue(str(matrix[2][2]))
        self.text.SetValue(str(matrix.tolist()))

    def OnSaveFile(self, _):
        """
        Saves the matrix in ASCII editor or 9 entried elements on Hard Disk
        """
        self.last_name_saved = self.filenamesave.GetValue()

        if self.last_name_saved:

            try:
                # radio button on text ascii editor
                if self.rbeditor.GetValue():
                    f = open(self.last_name_saved, "w")
                    text = self.text.GetValue()
                    f.write(text)
                    f.close()
                # 9 elements txtctrl editors
                else:
                    m11 = float(self.mat_a11.GetValue())
                    m12 = float(self.mat_a12.GetValue())
                    m13 = float(self.mat_a13.GetValue())

                    m21 = float(self.mat_a21.GetValue())
                    m22 = float(self.mat_a22.GetValue())
                    m23 = float(self.mat_a23.GetValue())

                    m31 = float(self.mat_a31.GetValue())
                    m32 = float(self.mat_a32.GetValue())
                    m33 = float(self.mat_a33.GetValue())

                    _file = open(self.last_name_saved, "w")
                    text = ("[[%.17f,%.17f,%.17f],\n[%.17f,%.17f,%.17f],\n[%.17f,%.17f,%.17f]]"
                        % (m11, m12, m13, m21, m22, m23, m31, m32, m33))
                    _file.write(text)
                    _file.close()

                self.statusbar.SetStatusText(
                    os.path.basename(self.last_name_saved) + " saved", 0)
                self.modify = False
                self.statusbar.SetStatusText("", 1)

                fullname = os.path.join(os.getcwd(), self.last_name_saved)
                wx.MessageBox("Matrix saved in %s" % fullname, "INFO")

            except IOError as error:

                dlg = wx.MessageDialog(self, "Error saving file\n" + str(error))
                dlg.ShowModal()
        else:
            print("No name input")

    def OnStoreMatrix(self, _):
        r"""
        Stores the matrix from the ASCII editor or the 9 entried elements
        in main list of orientation matrix for further simulation
        """
        self.last_name_stored = self.filenamestore.GetValue()

        if self.last_name_stored:

            # read 9 elements
            if not self.rbeditor.GetValue():
                m11 = float(self.mat_a11.GetValue())
                m12 = float(self.mat_a12.GetValue())
                m13 = float(self.mat_a13.GetValue())

                m21 = float(self.mat_a21.GetValue())
                m22 = float(self.mat_a22.GetValue())
                m23 = float(self.mat_a23.GetValue())

                m31 = float(self.mat_a31.GetValue())
                m32 = float(self.mat_a32.GetValue())
                m33 = float(self.mat_a33.GetValue())

                self.parent.dict_Rot[self.last_name_stored] = [
                    [m11, m12, m13],
                    [m21, m22, m23],
                    [m31, m32, m33]]

            # read ASCII editor
            else:
                paramraw = str(self.text.GetValue())

                listval = re.split("[ ()\[\)\;\,\]\n\t\a\b\f\r\v]", paramraw)
                listelem = []
                for elem in listval:
                    try:
                        val = float(elem)
                        listelem.append(val)
                    except ValueError:
                        continue

                nbval = len(listelem)
                if nbval != 9:
                    txt = "Something wrong, I can't read this matrix %s \n."
                    txt += "It doesn't contain 9 elements with float type ..."%paramraw
                    print(txt)

                    wx.MessageBox(txt, "VALUE ERROR")
                    return

                mat = np.zeros((3, 3))
                ind_elem = 0
                for i in range(3):
                    for j in range(3):
                        floatval = listelem[ind_elem]
                        mat[i][j] = floatval
                        ind_elem += 1

                self.parent.dict_Rot[self.last_name_stored] = mat

            self.statusbar.SetStatusText(
                os.path.basename(self.last_name_stored) + " stored", 0)

        else:
            print("No name input")

    def OnOpenMatrixFile(self, _):
        wcd = "All files(*)|*|Matrix files(*.mat)|*.mat"
        _dir = os.getcwd()
        open_dlg = wx.FileDialog(
                                self,
                                message="Choose a file",
                                defaultDir=_dir,
                                defaultFile="",
                                wildcard=wcd,
                                style=wx.OPEN | wx.CHANGE_DIR)
        if open_dlg.ShowModal() == wx.ID_OK:
            path = open_dlg.GetPath()

            try:
                _file = open(path, "r")
                paramraw = _file.read()
                _file.close()

                listval = re.split("[ ()\[\)\;\,\]\n\t\a\b\f\r\v]", paramraw)
                listelem = []
                for elem in listval:
                    try:
                        val = float(elem)
                        listelem.append(val)
                    except ValueError:
                        continue

                nbval = len(listelem)
                if nbval != 9:
                    txt = "Something wrong, I can't read this matrix %s \n."
                    txt += "It doesn't contain 9 elements with float type ..."%paramraw
                    print(txt)

                    wx.MessageBox(txt, "VALUE ERROR")
                    return

                mat = np.zeros((3, 3))
                ind_elem = 0
                for i in range(3):
                    for j in range(3):
                        floatval = listelem[ind_elem]
                        mat[i][j] = floatval
                        ind_elem += 1

                self.text.Clear()
                self.text.WriteText(str(mat))
                self.last_name_saved = path
                self.statusbar.SetStatusText("", 1)
                self.modify = False

            except IOError as error:
                dlg = wx.MessageDialog(self, "Error opening file\n" + str(error))
                dlg.ShowModal()

            except UnicodeDecodeError as error:
                dlg = wx.MessageDialog(self, "Error opening file\n" + str(error))
                dlg.ShowModal()

        open_dlg.Destroy()

    def EnterComboRot(self, evt):
        """
        in matrix editor
        """
        item = evt.GetSelection()
        self.CurrentMat = self.list_of_Rot[item]
        evt.Skip()

    def OnLookMatrix(self, _):
        matrix = self.parent.dict_Rot[self.CurrentMat]
        print("%s is :" % self.CurrentMat)
        print(matrix)

        self.mat_a11.SetValue(str(matrix[0][0]))
        self.mat_a12.SetValue(str(matrix[0][1]))
        self.mat_a13.SetValue(str(matrix[0][2]))
        self.mat_a21.SetValue(str(matrix[1][0]))
        self.mat_a22.SetValue(str(matrix[1][1]))
        self.mat_a23.SetValue(str(matrix[1][2]))
        self.mat_a31.SetValue(str(matrix[2][0]))
        self.mat_a32.SetValue(str(matrix[2][1]))
        self.mat_a33.SetValue(str(matrix[2][2]))
        self.text.SetValue(str(matrix))

    def OnLoadXMAS_INDfile(self, _):
        """
        old and may be obsolete indexation file reading procedure ?!
        """
        wcd = "All files(*)|*|Indexation files(*.ind)|*.ind|StrainRefined files(*.str)|*.str"
        _dir = os.getcwd()
        open_dlg = wx.FileDialog(
                                self,
                                message="Choose a file",
                                defaultDir=_dir,
                                defaultFile="",
                                wildcard=wcd,
                                style=wx.OPEN | wx.CHANGE_DIR)
        if open_dlg.ShowModal() == wx.ID_OK:
            path = open_dlg.GetPath()

            try:
                _file = open(path, "r")
                alllines = _file.read()
                _file.close()
                listlines = alllines.splitlines()

                if path.split(".")[-1] in ("IND", "ind"):

                    posmatrix = -1
                    lineindex = 0
                    for line in listlines:
                        if line[:8] == "matrix h":
                            print("Get it!: ", line)
                            posmatrix = lineindex
                            break
                        lineindex += 1

                    if posmatrix != -1:
                        print("firstrow", listlines[posmatrix + 1])
                        firstrow = np.array(
                            listlines[posmatrix + 1].split(), dtype=float)
                        secondrow = np.array(
                            listlines[posmatrix + 2].split(), dtype=float)
                        thirdrow = np.array(
                            listlines[posmatrix + 3].split(), dtype=float)
                        mat = np.array([firstrow, secondrow, thirdrow])
                        # indexation file contains matrix
                        # such as U G = Q  G must be normalized ??
                    else:
                        print("Did not find matrix in this ind file!...")

                    if self.text.GetLastPosition():
                        self.text.Clear()

                    self.text.WriteText(str(mat.tolist()))
                    self.last_name_saved = path
                    self.statusbar.SetStatusText("", 1)
                    self.modify = False

                elif path.split(".")[-1] in ("STR", "str"):
                    posmatrix = -1
                    lineindex = 0
                    for line in listlines:
                        if line[:17] == "coordinates of a*":
                            print("Get it!: ", line)
                            posmatrix = lineindex
                            break
                        lineindex += 1

                    if posmatrix != -1:
                        print("firstrow", listlines[posmatrix + 1])
                        firstrow = np.array(
                            listlines[posmatrix + 1].split(), dtype=float)
                        secondrow = np.array(
                            listlines[posmatrix + 2].split(), dtype=float)
                        thirdrow = np.array(
                            listlines[posmatrix + 3].split(), dtype=float)
                        mat = np.array([firstrow, secondrow, thirdrow])
                        # contrary to .ind file, matrix is transposed!!
                        #   UB G  = Q  with G normalized
                        mat = np.transpose(mat)
                    else:
                        print("Did not find matrix in this str file!...")

                    if self.text.GetLastPosition():
                        self.text.Clear()

                    self.text.WriteText(str(mat.tolist()))
                    self.last_name_saved = path
                    self.statusbar.SetStatusText("", 1)
                    self.modify = False

            except IOError as error:
                dlg = wx.MessageDialog(self, "Error opening file\n" + str(error))
                dlg.ShowModal()

            except UnicodeDecodeError as error:
                dlg = wx.MessageDialog(self, "Error opening file\n" + str(error))
                dlg.ShowModal()

        open_dlg.Destroy()

    def OnConvert(self, _):
        """ old functionality...
        """
        wx.MessageBox('This conversion has been deleted', 'Info')

        # text = str(self.text.GetValue())
        # tu = text.replace("[", "").replace("]", "")
        # ta = tu.split(",")
        # to = [float(elem) for elem in ta]
        # matrix = np.array([to[:3], to[3:6], to[6:]])

        # dlg = wx.MessageDialog(
        #     self,
        #     "New matrix will be displated in Text Editor Input\nConversion will use current calibration, namely xbet: %.4f \n  \nDo you want to continue ?"
        #     % self.parent.defaultParam[3])
        # if dlg.ShowModal() == wx.ID_OK:

        #     evt.Skip()

        #     # newmatrix = F2TC.matxmas_to_OrientMatrix(matrix, LaueToolsframe.defaultParam)
        #     newmatrix = FXL.convert_fromXMAS_toLaueTools(
        #         matrix, 5.6575, anglesample=40.0, xbet=self.parent.defaultParam[3])
        #     print("Matrix read in editor")
        #     print(matrix.tolist())
        #     print("Matrix as converted to Lauetools by F2TC")
        #     print(newmatrix.tolist())
        #     # matrix UB normalized
        #     # must be multiplied by lattic param*

        #     self.rbeditor.SetValue(True)
        #     self.text.SetValue(str(newmatrix.tolist()))
        # else:
        #     # TODO: advise how to set new calibration parameter(in calibration menu ?)
        #     evt.Skip()

    def OnConvertlabtosample(self, _):
        """
        qs= R ql    q expressed in sample frame(xs,ys,zs) = R * ql
        with ql being q expressed in lab frame(x,y,z)
        G=ha*+kb*+lc*
        ql = UB * G with UB orientation and strain matrix
        ie UB columns are a*, b*, c* expressed in x,y,z frame
        qs = R * UB * G
        R*UB in the orientation matrix in sample frame
        ie columns are a*, b*,c* expressed in xs,ys,zs frame

        Gs = R Gl    G expressed in sample frame(xs,ys,zs)  =  R * Gl
        with Gl being G expressed in lab frame

        qs = R*UB*invR  Gs means that R*UB*invR is the orientation matrix .
        From a*,b*,c* in xs,ys,zs to q in xs,ys,zs
        """
        text = str(self.text.GetValue())
        tu = text.replace("[", "").replace("]", "")
        ta = tu.split(",")
        to = [float(elem) for elem in ta]
        UB = np.array([to[:3], to[3:6], to[6:]])

        if not self.invconvert.GetValue():  # direct conversion
            anglesample = 40.0 * DEG
        else:  # inverse conversion
            anglesample = -40.0 * DEG

        Rot = np.array([[np.cos(anglesample), 0, np.sin(anglesample)],
                        [0, 1, 0],
                        [-np.sin(anglesample), 0, np.cos(anglesample)]])
        invRot = np.linalg.inv(Rot)
        UBs = np.dot(Rot, UB)
        print("UB as read in editor")
        print(UB.tolist())
        print("UB converted in lauetools sample frame(From G=ha*+kb*+lc* with a*,b* and c* "
            "expressed in lab frame to q expressed in sample frame)")
        print(UBs.tolist())
        print("UB converted in XMAS-like sample frame")
        print(np.dot(np.array([[0, 1, 0], [-1, 0, 0], [0, 0, 1]]), UBs).tolist())
        print("UB converted in lauetools sample frame(From G=ha*+kb*+lc* with a*,b* "
            "and c* expressed in sample frame to q expressed in sample frame)")
        print(np.dot(UBs, invRot).tolist())

        # matrix UB normalized
        # must be multiplied by lattic param*

        self.rbeditor.SetValue(True)
        self.text.SetValue(str(UBs.tolist()))

    def OnQuit(self, evt):
        dlg = wx.MessageDialog(self,
            'To use stored Matrices in simulation boards that do not appear, click '
            'on "refresh choices" button before.')
        if dlg.ShowModal() == wx.ID_OK:
            self.Close()
            evt.Skip()
        else:
            evt.Skip()



r"""
GUI for microdiffraction Laue Pattern peaks list file loading.

This module belongs to the open source LaueTools project
with a free code repository at at gitlab.esrf.fr

(former version with python 2.7 at https://sourceforge.net/projects/lauetools/)

or for python3 and 2 in

https://gitlab.esrf.fr/micha/lauetools

J. S. Micha February 2020
mailto: micha --+at-+- esrf --+dot-+- fr
"""
import os
import sys
import copy

import wx

if sys.version_info.major == 3:
    from . import CCDFileParametersGUI as CCDParamGUI
    from . import DetectorParameters as DP
    from .. import dict_LaueTools as DictLT
    from .. import LaueGeometry as F2TC
    from .. import IOLaueTools as IOLT
else:
    import GUI.CCDFileParametersGUI as CCDParamGUI
    import GUI.DetectorParameters as DP
    import dict_LaueTools as DictLT
    import LaueGeometry as F2TC
    import IOLaueTools as IOLT

DICT_LAUE_GEOMETRIES = DictLT.DICT_LAUE_GEOMETRIES

DICT_LAUE_GEOMETRIES_INFO = DictLT.DICT_LAUE_GEOMETRIES_INFO

if wx.__version__ < "4.":
    WXPYTHON4 = False
else:
    WXPYTHON4 = True
    wx.OPEN = wx.FD_OPEN

def defaultFileDialogOptions(dirname):
    """ Return a dictionary with file dialog options that can be
        used in both the save file dialog as well as in the open
        file dialog. """
    wcd = "All files(*)|*|cor file(*.cor)|*.cor|peaks list(*.dat)|*.dat"
    return dict(message="Choose a data file (peaks list)",
                defaultDir=dirname,
                wildcard=wcd)

def askUserForFilename(parent, **dialogOptions):
    """
    provide a dialog to browse the folders and files

    set self.dirname, self.filename
    returns boolean userProvidedFilename
    """
    dialog = wx.FileDialog(parent, **dialogOptions)
    if dialog.ShowModal() == wx.ID_OK:
        userProvidedFilename = True
        # self.filename = dialog.GetFilename()
        # #self.dirname = dialog.GetDirectory()

        allpath = dialog.GetPath()
        print(allpath)

        parent.dirname, parent.filename = os.path.split(allpath)

    else:
        userProvidedFilename = False
    dialog.Destroy()
    return userProvidedFilename

def OpenCorfile(filename, parent):
    """
    Read a .cor file with 5 columns 2theta chi pixX pixY I
    reads detector parameters and set defaultParam according to them
    returns self.data_theta, self.data_chi, self.data_pixX, self.data_pixY, self.data_I

    creates or updates self.Current_peak_data (all columns of .cor file)
    creates or updates self.data_theta, self.data_chi, self.data_I, self.data_pixX,
        self.data_pixY, self.CCDCalibDict

    parent must have attributes: kf_direction_from_file, CCDLabel, detectordiameter
    """
    kf_direction_from_file, CCDLabel = parent.kf_direction_from_file, parent.CCDLabel

    print('Opening %s'%filename)
    (Current_peak_data,
        data_theta,
        data_chi,
        data_pixX,
        data_pixY,
        data_I,
        calib,
        CCDCalibDict,
    ) = IOLT.readfile_cor(filename, output_CCDparamsdict=True)

    print("\nCCDCalibDict after readfile_cor ", CCDCalibDict)
    if 'CCDLabel' in CCDCalibDict:
        CCDLabel = CCDCalibDict['CCDLabel']

    CheckCCDCalibParameters(CCDCalibDict, kf_direction_from_file, CCDLabel, parent)

    pixelsize_fromfile = IOLT.getpixelsize_from_corfile(filename)

    # update  CCDLabel,   framedim (nb pixels * nb pixels)
    parent.CCDLabel = CCDCalibDict['CCDLabel']
    print('parent.CCDLabel', parent.CCDLabel)
    parent.framedim = DictLT.dict_CCD[parent.CCDLabel][0]
    parent.pixelsize = DictLT.dict_CCD[parent.CCDLabel][0]

    # set parent parameters:

    if pixelsize_fromfile:
        parent.pixelsize = pixelsize_fromfile
        print('reading pixelsize from file: %f mm'%parent.pixelsize)
    else:
        parent.pixelsize = DictLT.dict_CCD[parent.CCDLabel][1]
        print('reading pixelsize from CCDLabel : %f mm'%parent.pixelsize)
    if calib is not None:
        parent.defaultParam = calib

    parent.data_XY = (data_pixX, data_pixY)

    (parent.Current_peak_data,
        parent.data_theta,
        parent.data_chi,
        parent.data_pixX,
        parent.data_pixY,
        parent.data_I,
        calib,
        parent.CCDCalibDict) = (Current_peak_data,
                        data_theta,
                        data_chi,
                        data_pixX,
                        data_pixY,
                        data_I,
                        calib,
                        CCDCalibDict)

    # Spots List to index object ----------------------
    print("self.CCDCalibDict  end ", parent.CCDCalibDict)
    print("self.pixelsize  end", parent.pixelsize)

    return (data_theta, data_chi,
            data_pixX, data_pixY, data_I)

def CheckCCDCalibParameters(CCDCalibDict, kf_direction_from_file, CCDLabel, parent):
    """
    check if all CCD parameters are read from file .cor

    parent must have attributes:
        - kf_direction
    """
    ccp = DictLT.CCD_CALIBRATION_PARAMETERS

    sorted_list_parameters = [ccp[7], ccp[10]]
    for key in sorted_list_parameters:
        if key not in CCDCalibDict:
            missing_param = key

            if (missing_param == "kf_direction" and kf_direction_from_file is None):

                LaueGeomBoard = SetGeneralLaueGeometry(parent, -1, "Select Laue Geometry")
                LaueGeomBoard.ShowModal()
                LaueGeomBoard.Destroy()

            if missing_param == "CCDLabel" or CCDLabel is None:
                DPBoard = CCDParamGUI.CCDFileParameters(parent, -1, "CCD File Parameters Board",
                                                    CCDLabel)
                DPBoard.ShowModal()
                DPBoard.Destroy()

def Launch_DetectorParamBoard(parent):
    """Board to enter manually detector params
    Launch Entry dialog
    """
    Parameters_dict = {}
    Parameters_dict["CCDLabel"] = parent.CCDLabel
    Parameters_dict["CCDParam"] = parent.defaultParam
    Parameters_dict["pixelsize"] = parent.pixelsize
    Parameters_dict["framedim"] = parent.framedim
    Parameters_dict["detectordiameter"] = parent.detectordiameter
    Parameters_dict["kf_direction"] = parent.kf_direction
    # print "old param",self.defaultParam+[self.pixelsize]
    DPBoard = DP.DetectorParameters(parent, -1, "Detector parameters Board", Parameters_dict)
    DPBoard.ShowModal()
    DPBoard.Destroy()


def OnOpenPeakList(parent):
    """
    Load Peak list data (.dat or .cor)

    set parent attributes:
        - dirname
        - filename   .dat file or .cor file (built from .dat file)
        - defaultParam
        - pixelsize
        - framedim
        - detectordiameter
        - kf_direction
        - kf_direction_from_file
        - PeakListDatFileName  .dat file
        - DataPlot_filename
    """
    if askUserForFilename(parent, style=wx.OPEN, **defaultFileDialogOptions(parent.dirname)):
        # print "Current directory in OnOpenPeakList()",self.dirname
        os.chdir(parent.dirname)

        # print String_in_File_Data # in stdout/stderr
        DataPlot_filename = str(parent.filename)
        print("Current file   :", DataPlot_filename)

        prefix, file_extension = DataPlot_filename.rsplit(".", 1)

    if file_extension in ("dat", "DAT"):

        # open .det file to compute 2thea and chi scattering angles and write .cor file
        # will set defaultParam pixelsize framedim detectordiameter kf_direction
        Launch_DetectorParamBoard(parent)

        # will set kf_direction attributr
        LaueGeomBoard = SetGeneralLaueGeometry(parent, -1, "Select Laue Geometry")
        LaueGeomBoard.ShowModal()
        LaueGeomBoard.Destroy()

        print("kf_direction in OnOpenPeakList", parent.kf_direction)

        # compute 2theta and chi according to detector calibration geometry
        (twicetheta, chi, dataintensity,
            data_x, data_y) = F2TC.Compute_data2thetachi(
                                                DataPlot_filename,
                                                (0, 1, 3),
                                                1,
                                                sorting_intensity="yes",
                                                param=parent.defaultParam,
                                                pixelsize=parent.pixelsize,
                                                kf_direction=parent.kf_direction)
        # write .cor file
        IOLT.writefile_cor("dat_" + prefix,
                            twicetheta,
                            chi,
                            data_x,
                            data_y,
                            dataintensity,
                            sortedexit=0,
                            param=parent.defaultParam + [parent.pixelsize],
                            initialfilename=DataPlot_filename,
                        )  # check sortedexit = 0 or 1 to have decreasing intensity sorted data

        print("%s has been created with defaultparameter" % ("dat_" + prefix + ".cor"))
        print("%s" % str(parent.defaultParam))

        parent.PeakListDatFileName = copy.copy(DataPlot_filename)
        parent.kf_direction_from_file = parent.kf_direction

        file_extension = "cor"
        DataPlot_filename = "dat_" + prefix + "." + file_extension
        # WARNING: this file will be read in the next "if" clause

    # for .cor file ------------------------------
    if file_extension == "cor":
        # read peak list and detector calibration parameters
        OpenCorfile(DataPlot_filename, parent)

        parent.DataPlot_filename = DataPlot_filename
        parent.filename = parent.DataPlot_filename



# --- -------------------- general Laue Geometry settings
class SetGeneralLaueGeometry(wx.Dialog):
    """
    Dialog Class to set  general Laue Geometry

    parent must have   kf_direction attribute
    """
    def __init__(self, parent, _id, title):

        wx.Dialog.__init__(self, parent, _id, title, size=(400, 200))

        self.parent = parent

        txt = wx.StaticText(self, -1, "Choose Laue Geometry")
        font = wx.Font(16, wx.DEFAULT, wx.NORMAL, wx.BOLD)
        txt.SetFont(font)
        txt.SetForegroundColour((255, 0, 0))

        initialGeo = DICT_LAUE_GEOMETRIES[parent.kf_direction]

        self.combogeo = wx.ComboBox(self, -1, str(initialGeo), size=(-1, 40),
                                    choices=["Top Reflection (2theta=90)",
                                            "Transmission (2theta=0)",
                                            "Back Reflection (2theta=180)"],
                                    style=wx.CB_READONLY)

        self.combogeo.Bind(wx.EVT_COMBOBOX, self.OnChangeGeom)

        txtinfo = wx.StaticText(self, -1, "Infos :  ")

        self.comments = wx.TextCtrl(self, style=wx.TE_MULTILINE, size=(300, 50))
        self.comments.SetValue(str(DictLT.DICT_LAUE_GEOMETRIES_INFO[initialGeo]))

        btna = wx.Button(self, 1, "Accept", size=(150, 40))
        btna.Bind(wx.EVT_BUTTON, self.OnAccept)
        btna.SetDefault()

        btnc = wx.Button(self, 2, "Cancel", size=(100, 40))
        btnc.Bind(wx.EVT_BUTTON, self.OnQuit)

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(btna, 1)
        hbox.Add(btnc, 1)

        h2box = wx.BoxSizer(wx.HORIZONTAL)
        h2box.Add(txtinfo, 0)
        h2box.Add(self.comments, 0)

        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(txt, 0, wx.EXPAND)
        vbox.Add(self.combogeo, 0, wx.EXPAND)
        vbox.Add(hbox, 0)
        vbox.Add(wx.StaticText(self, -1, ""), 0)
        vbox.Add(h2box, 0, wx.EXPAND)

        self.SetSizer(vbox)

    def OnChangeGeom(self, _):
        """change detection geometry
        """
        focus_geom = self.combogeo.GetValue()

        #         print "Laue Geometry info :", DICT_LAUE_GEOMETRIES_INFO[focus_geom]
        self.comments.SetValue(str(DictLT.DICT_LAUE_GEOMETRIES_INFO[focus_geom]))

    #         self.sb.SetStatusText(str(DICT_LAUE_GEOMETRIES_INFO[focus_geom]))

    def OnAccept(self, _):
        """accept geometry
        """
        LaueGeometry = self.combogeo.GetValue()

        if LaueGeometry == "Transmission (2theta=0)":
            kf_direction = "X>0"
        elif LaueGeometry == "Top Reflection (2theta=90)":
            kf_direction = "Z>0"
        elif LaueGeometry == "Back Reflection (2theta=180)":
            kf_direction = "X<0"

        self.parent.kf_direction = kf_direction

        print("Laue geometry set to: %s" % LaueGeometry)
        print("kf_direction set to: %s" % kf_direction)

        self.Close()

    def OnQuit(self, _):
        """ quit """
        self.Close()
